Package: tfevents
Title: Write Events for 'TensorBoard'
Version: 0.0.5
Authors@R: c(
    person("Daniel", "Falbel", email = "dfalbel@gmail.com", role = c("aut", "cph")),
    person("Tomasz", "Kalinowski", email = "tomasz@posit.co", role = c("cre")),
    person(family = "Posit, PBC", role = c("cph")),
    person(family = "The tl::optional authors", role = c("cph"), comment = "For the vendored tl::optional code."),
    person("Mark", "Adler", role = c("cph"), comment = "For the included crc32c code.")
    )
Description: Provides a convenient way to log scalars, images, audio, and histograms in the 'tfevent' record file format. 
  Logged data can be visualized on the fly using 'TensorBoard', a web based tool that focuses on visualizing the training 
  progress of machine learning models.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
LinkingTo: Rcpp
Imports: Rcpp, withr, fs, rlang (>= 1.0.0), vctrs, blob, png, digest,
        cli, zeallot
Suggests: testthat (>= 3.0.0), tibble, tidyr, reticulate, rmarkdown,
        ggplot2, tensorflow, wav
SystemRequirements: libprotobuf, protobuf-compiler
URL: https://github.com/mlverse/tfevents,
        https://mlverse.github.io/tfevents/
BugReports: https://github.com/mlverse/tfevents/issues
Config/testthat/edition: 3
Config/Needs/website: tensorflow, webshot2, keras
NeedsCompilation: yes
Packaged: 2026-04-27 11:40:43 UTC; dfalbel
Author: Daniel Falbel [aut, cph],
  Tomasz Kalinowski [cre],
  Posit, PBC [cph],
  The tl::optional authors [cph] (For the vendored tl::optional code.),
  Mark Adler [cph] (For the included crc32c code.)
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-04-27 12:40:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-27 21:19:21 UTC; unix
Archs: tfevents.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(as_event,character)
S3method(as_event,list)
S3method(as_event,numeric)
S3method(as_event,tfevents_summary_values)
S3method(as_tensor_proto,array)
S3method(as_tensor_proto,blob)
S3method(as_tensor_proto,character)
S3method(as_tensor_proto,list)
S3method(format,tfevents_event)
S3method(format,tfevents_summary)
S3method(format,tfevents_summary_values)
S3method(summary_audio,array)
S3method(summary_audio,blob)
S3method(summary_audio,raw)
S3method(summary_histogram,array)
S3method(summary_histogram,numeric)
S3method(summary_image,array)
S3method(summary_image,blob)
S3method(summary_image,ggplot)
S3method(summary_image,raw)
S3method(summary_text,character)
S3method(value,tfevents_summary_values)
S3method(value,tfevents_summary_values_audio)
S3method(value,tfevents_summary_values_histograms)
S3method(value,tfevents_summary_values_images)
S3method(value,tfevents_summary_values_scalars)
S3method(value,tfevents_summary_values_text)
S3method(vec_cast,hparams_experiment.hparams_experiment)
S3method(vec_cast,hparams_hparam_info.hparams_hparam_info)
S3method(vec_cast,hparams_interval.hparams_interval)
S3method(vec_cast,hparams_session_start_info.hparams_session_start_info)
S3method(vec_cast,list.hparams_hparams_plugin_data)
S3method(vec_cast,summary_summary_image.summary_summary_image)
S3method(vec_cast,tensor_proto.tensor_proto)
S3method(vec_cast,tfevents_event.tfevents_event)
S3method(vec_cast,tfevents_summary.tfevents_summary_values)
S3method(vec_cast,tfevents_summary_values.tfevents_summary_values)
S3method(vec_ptype2,hparams_experiment.hparams_experiment)
S3method(vec_ptype2,hparams_hparam_info.hparams_hparam_info)
S3method(vec_ptype2,hparams_hparams_plugin_data.list)
S3method(vec_ptype2,hparams_interval.hparams_interval)
S3method(vec_ptype2,hparams_session_start_info.hparams_session_start_info)
S3method(vec_ptype2,summary_hparams_config.tfevents_summary_values)
S3method(vec_ptype2,summary_summary_image.summary_summary_image)
S3method(vec_ptype2,tfevents_event.tfevents_event)
S3method(vec_ptype2,tfevents_summary.tfevents_summary_values)
S3method(vec_ptype2,tfevents_summary_scalar.tfevents_summary_values)
S3method(vec_ptype2,tfevents_summary_values.summary_hparams_config)
S3method(vec_ptype2,tfevents_summary_values.tfevents_summary)
S3method(vec_ptype2,tfevents_summary_values.tfevents_summary_scalar)
S3method(vec_ptype2,tfevents_summary_values.tfevents_summary_values)
export(as_event)
export(collect_events)
export(events_logdir)
export(get_default_logdir)
export(get_global_step)
export(hparams_hparam)
export(hparams_metric)
export(local_logdir)
export(log_event)
export(log_hparams)
export(log_hparams_config)
export(set_default_logdir)
export(set_global_step)
export(summary_audio)
export(summary_histogram)
export(summary_hparams)
export(summary_hparams_config)
export(summary_image)
export(summary_metadata)
export(summary_scalar)
export(summary_text)
export(value)
export(with_logdir)
import(vctrs)
importFrom(Rcpp,sourceCpp)
importFrom(blob,blob)
importFrom(utils,tail)
importFrom(zeallot,"%<-%")
useDynLib(tfevents, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# tfevents

<!-- badges: start -->

[![R-CMD-check](https://github.com/mlverse/tfevents/actions/workflows/check.yaml/badge.svg)](https://github.com/mlverse/tfevents/actions/workflows/check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/mlverse/tfevents/branch/main/graph/badge.svg)](https://app.codecov.io/gh/mlverse/tfevents?branch=main)
[![CRAN
status](https://www.r-pkg.org/badges/version/tfevents)](https://CRAN.R-project.org/package=tfevents)
[![](https://cranlogs.r-pkg.org/badges/tfevents)](https://cran.r-project.org/package=tfevents)
<!-- badges: end -->

tfevents allows logging data from machine learning experiments to a file
format that can be later consumed by
[TensorBoard](https://www.tensorflow.org/tensorboard) in order to
generate visualizations.

## Installation

You can install tfevents from CRAN with:

``` r
install.packages("tfevents")
```

You can install the development version of tfevents from
[GitHub](https://github.com/) with:

You need to have `cmake` on your path. See installation instructions in
the [cmake install webpage](https://cmake.org/resources/) - or:

If you use `brew` on MacOS you can run:

``` shell
brew install cmake
```

Or on linux install the `cmake` library, for example on Debian systems:

``` shell
sudo apt install cmake
```

``` r
# install.packages("devtools")
devtools::install_github("mlverse/tfevents")
```

## Example

The main entrypoint in tfevents API is the `log_event` function. It can
be used to log **summaries** like scalars, images, audio (Coming soon),
histograms (Coming soon) and arbitrary tensors (soon) to a log
directory, which we like to call `logdir`. You can later point
TensorBoard to this `logdir` to visualize the results.

``` r
library(tfevents)
```

Summaries are always associated to a step in the TensorBoard API, and
`log_event` automatically increases the **`step`** everytime it’s
called, unless you provide the `step` argument.

Let’s start by logging some metrics:

``` r
epochs <- 10
for (i in seq_len(epochs)) {
  # training code would go here
  log_event(
    train = list(loss = runif(1), acc = runif(1)),
    valid = list(loss = runif(1), acc = runif(1))
  )
}
```

By default this will create a `logs` directory in your working directory
and write metrics to it - you can change the default `logdir` using
context like `with_logdir` or globally with `set_default_logdir()`.

Since we passed a nested list of metrics, `log_event` will create
subdirectories under `logs` to write metrics for each group.

``` r
fs::dir_tree("logs")
#> logs
#> ├── train
#> │   └── events.out.tfevents.1719410709.v2
#> └── valid
#>     └── events.out.tfevents.1719410709.v2
```

You can later point TensorBoard to that logdir using TensorBoard’s
command line interface or tensorflow’s utility function `tensorboard()`

``` r
tensorflow::tensorboard(normalizePath("logs"), port = 6060)
#> Started TensorBoard at http://127.0.0.1:6060
```

TensorBoard will display the results in a dashbboard, similar to one you
can see in the screenshot below:

<img src="man/figures/README-tensorboard-1.png" width="100%" />

You can learn more on the [tfevents website](https://mlverse.github.io/tfevents/).

#' Collect data from tfevents records
#'
#' Collects all events of a kind in a single data.frame ready for analysis.
#'
#' @param logdir The log directory that you want to query events from. Either a
#'  file path or a connection created with [events_logdir()].
#' @param n The maximum number of events to read from the connection. If `NULL`
#'  then all events are read, the default is `NULL`.
#' @param type The kind of events that are to be read. By default all events are
#'  read. If a different type is specified, then the result can include other
#'  columns as well as more lines.
#'
#' @returns
#' A `tibble` with the collected events.
#'
#' @examples
#' temp <- tempfile()
#' with_logdir(temp, {
#'   for(i in 1:5) {
#'     log_event(my_log = runif(1))
#'   }
#' })
#' # collect all events in files, including file description events
#' collect_events(temp)
#' # collect summaries in the logdir
#' collect_events(temp, type = "summary")
#' # collect only scalar events
#' collect_events(temp, type = "scalar")
#'
#' @export
collect_events <- function(logdir = get_default_logdir(), n = NULL,
                           type = c("any", "summary", "scalar")) {
  logdir <- events_logdir(logdir)
  type <- rlang::arg_match(type)

  if (!is.null(n) && n <= 0) {
    cli::cli_abort(c(
      "{.arg n} must be positive or `NULL`",
      "{.val {n}} is <=0 and not allowed."
    ))
  }

  events <- list()
  repeat {
    event <- read_next(logdir, type = type)
    if (is_exhausted(event)) {
      break
    } else {
      events[[length(events) + 1]] <- event
      if (!is.null(n) && length(events) == n)
        break
    }
  }
  vec_rbind(!!!events)
}

#' @describeIn collect_events Creates a connection to a logdir that can be reused
#'   to read further events later.
#' @export
events_logdir <- function(logdir = get_default_logdir()) {
  if (inherits(logdir, "tfevents_logdir_connection"))
    return(logdir)
  new_events_logdir_connection(logdir)
}

new_events_logdir_connection <- function(logdir) {
  con <- structure(
    new.env(parent = emptyenv()),
    class = "tfevents_logdir_connection"
  )

  files <- fs::dir_ls(logdir, type = "file", regexp = ".*tfevents.*", recurse = TRUE)
  iterators <- create_iterators(files, logdir)

  con$logdir <- logdir
  con$iterators <- iterators
  con$files <- files

  con
}

# possibly includes new files that might have been created in the logdir, and
# could yield new values.
refresh_events_logdir_connection <- function(con) {
  # if exhausted, maybe there's a new file in the directory that we were not
  # tracking yet, so we try it before returing the exhausted flag.
  # check if new files were added to the directory
  new_files <- fs::dir_ls(con$logdir, type = "file", regexp = ".*tfevents.*", recurse = TRUE)
  new_files <- new_files[!new_files %in% con$files]

  # append to files and iterators
  con$files <- c(con$files, new_files)
  con$iterators <- append(con$iterators, create_iterators(new_files, con$logdir))
  invisible(NULL)
}

read_next <- function(con, type) {
  if (type == "any") {
    read_next_event(con)
  } else if (type == "summary") {
    read_next_summary(con)
  } else if (type == "scalar") {
    read_next_scalar(con)
  }
}

read_next_event <- function(con) {
  out <- try_iterators(con$iterators)
  if (is_exhausted(out)) {
    refresh_events_logdir_connection(con)
    out <- try_iterators(con$iterators)
  }
  out
}

read_next_summary <- function(con) {
  out <- read_next_event(con)
  if (is_exhausted(out)) {
    out
  } else if (is.na(out$summary)) {
    read_next_summary(con)
  } else {
    out <- tidyr::unnest(out, summary)
    out$tag <- field(out$summary, "tag")
    out$plugin <- plugin(out$summary)
    out
  }
}

read_next_scalar <- function(con) {
  out <- read_next_summary(con)
  if (is_exhausted(out)) {
    out
  } else if (out$plugin == "scalars") {
    out$value <- value(out$summary)
    out
  } else {
    read_next_scalar(con)
  }
}

try_iterators <- function(iterators) {
  rlang::check_installed("tibble")
  for (iterator in iterators) {
    event <- try(event_file_iterator_next(iterator), silent = TRUE)
    if (!inherits(event, "try-error")) {
      events <- tibble::tibble(event = event)
      events$run <- field(events$event, "run")
      events$step <- field(events$event, "step")
      events$summary <- field(events$event, "summary")
      return(events)
    }
  }
  exhausted()
}

#' Extracts the value of a summary value
#'
#' Summaries are complicated objects because they reflect the Protobuf object
#' structure that are serialized in the tfevents records files. This function
#' allows one to easily query vaues from summaries and will dispatch to the
#' correct way to extract images, audio, text, etc from summary values.
#'
#' @param x A `tfevents_summary_values` object.
#' @param ... Currently unused. To allow future extension.
#' @returns
#' Depending on the type of the summary it returns an image, audio, text or
#' scalar.
#'
#' @examples
#' temp <- tempfile()
#' with_logdir(temp, {
#'   for(i in 1:5) {
#'     log_event(my_log = runif(1))
#'   }
#' })
#'
#' # iterate over all events
#' summary <- collect_events(temp, n = 1, type = "summary")
#' value(summary$summary)
#'
#' @export
value <- function(x, ...) {
  UseMethod("value")
}

#' @describeIn value Acess values from `summary_values`.
#' @param as_list A boolean indicating if the results should be returned in a list.
#'   The default is to return a single value. If you need to extract values from
#'   multiple summaries use `as_list = TRUE`.
#' @export
value.tfevents_summary_values <- function(x, ..., as_list = FALSE) {
  if (!vec_size(x) == 1 && !as_list) {
    cli::cli_abort(c(
      "You must pass a single summary_value to get it's value.",
      i = "Got size {.val {vec_size(x)}}",
      i = "Use {.val as_list = TRUE} to cast many values at once."
    ))
  }
  if (as_list) {
    lapply(x, value)
  } else {
    class(x) <- c(paste0("tfevents_summary_values_", plugin(x)), class(x))
    value(x)
  }
}

#' @export
value.tfevents_summary_values_scalars <- function(x, ...) {
  field(x, "value")
}

#' @export
value.tfevents_summary_values_histograms <- function(x, ...) {
  tensor <- field(x, "tensor")

  values <- field(tensor, "content")[[1]]
  dims <- field(field(tensor, "shape"), "dim")[[1]]

  # values in the tensor arre in C ordering, so we need to move to fortran ordering.
  arr <- aperm(array(values, dim = rev(dims)), perm = rev(seq_along(dims)))

  df <- as.data.frame(arr)
  colnames(df) <- c("lower", "upper", "count")

  df
}

#' @export
value.tfevents_summary_values_audio <- function(x, ...) {
  # audio is stored in the tensor field.
  tensor <- field(x, "tensor")

  # content is serialized as a wave encoded tensor.
  content <- field(tensor, "content")[[1]]

  # in order to read, we first write the bytes to a file, then read with wav::read_wav
  tmp <- tempfile(fileext = ".wav")
  on.exit({file.remove(tmp)}, add = TRUE)
  writeBin(object = content[[1]], con = tmp)

  wav::read_wav(tmp)
}

#' @export
value.tfevents_summary_values_images <- function(x, ...) {
  # images are stored as tensors in the summary proto
  image <- field(x, "tensor")

  # the stored tensor is a 1d vector with 3 elements.
  # first 2 describe the dimension of the image, the third
  # contains a PNG encoded image.
  buffer <- field(image, "content")[[1]]
  dim1 <- readBin(buffer[[1]], what = integer(), size = 1)
  dim2 <- readBin(buffer[[2]], what = integer(), size = 1)
  img <- png::readPNG(buffer[[3]])

  if (!identical(dim(img)[1:2], c(dim1, dim2))) {
    cli::cli_abort("An error ocurred. Report a issue in the tfevents repository.")
  }

  img
}

#' @export
value.tfevents_summary_values_text <- function(x, ...) {
  tensor <- field(x, "tensor")
  rawToChar(field(tensor, "content")[[1]][[1]])
}

plugin <- function(summary) {
  if (!inherits(summary, "tfevents_summary_values")) {
    cli::cli_abort(c(
      "{.arg summary} must be {.cls tfevents_summary_values}",
      i = "Got object with {.cls {class(summary)}}."
    ))
  }

  field(field(summary, "metadata"), "plugin_name")
}

exhausted <- function () {
  as.symbol(".__exhausted__.")
}

is_exhausted <- function (x) {
  identical(x, exhausted())
}

fill_run_field <- function(event, run) {
  field(event, "run") <- rep(run, vec_size(event))
  event
}

create_iterators <- function(files, logdir) {
  lapply(files, function(name) {
    create_event_file_iterator(
      name,
      fs::path_dir(fs::path_rel(name, logdir))
    )
  })
}

test_that("Can't have untagged summaries", {

  temp <- tempfile()

  expect_error(with_logdir(temp, {
    log_event(1)
  }), regexp = "must have a tag")

  expect_error(with_logdir(temp, {
    log_event(summary_scalar(1))
  }), regexp = "must have a tag")

  expect_error(with_logdir(temp, {
    log_event(summary_scalar(1, tag = "hello"))
  }), regexp = NA)

  scalars <- collect_events(temp, type = "scalar")
  expect_equal(scalars$tag, "hello")

})

test_that("collect float tensors", {
  temp <- tempfile()
  with_logdir(temp, {
   log_event(hello = summary_histogram(runif(1000)))
  })
  x <- collect_events(temp)

  expect_equal(nrow(x), 2)
})

test_that("can use the plugin function", {

  temp <- tempfile()
  with_logdir(temp, {
    log_event(hello = summary_histogram(runif(1000)))
    log_event(bye = 1)
  })

  summaries <- collect_events(temp, type = "summary")
  expect_equal(plugin(summaries$summary), c("histograms", "scalars"))
  expect_equal(summaries$plugin, c("histograms", "scalars"))

})

test_that("can iterate over events", {
  # tests low level reading functionality.

  get_scalar_value <- function(value) {
    field(field(value, "summary")[[1]], "value")
  }

  temp <- tempfile()
  with_logdir(temp, {
    log_event(hello = 1)
  })

  con <- events_logdir(temp)

  # reads the file definition event
  value <- collect_events(con, n = 1)
  expect_true(inherits(value, "tbl"))
  expect_true(inherits(value$event, "tfevents_event"))

  # reads the scalar
  value <- collect_events(con, n = 1)
  expect_true(inherits(value$event, "tfevents_event"))
  expect_equal(get_scalar_value(value$event), 1)

  # no more events in the file, so exhausted is returned
  value <- collect_events(con, n = 1)
  expect_true(nrow(value) == 0)

  with_logdir(temp, {
    log_event(hello = 2)
  })

  # new events are written to the same file,
  # read them
  value <- collect_events(con, n = 1)
  expect_true(inherits(value$event, "tfevents_event"))
  expect_equal(get_scalar_value(value$event), 2)

  # new events are written to a different file in the
  # directory and they are read.
  with_logdir(file.path(temp, "hello"), {
    log_event(hello = 3)
  })

  # first file event, is always a dummy event containg
  # timestamps and etc.
  value <- collect_events(con, n = 1)
  expect_true(inherits(value$event, "tfevents_event"))

  value <- collect_events(con, n = 1)
  expect_true(inherits(value$event, "tfevents_event"))
  expect_equal(get_scalar_value(value$event), 3)

  # no more events available, returns exhausted
  value <- collect_events(con, n = 1)
  expect_true(nrow(value) == 0)
})

test_that("can extract value", {
  skip_if_not_installed("wav")

  temp <- tempfile()

  orig_img <- png::readPNG(test_path("resources/img.png"))
  img <- array(orig_img, dim = c(1, 28, 28, 1))

  f <- wav::read_wav(test_path("resources/test-audio.wav"))
  audio <- array(t(f), dim = c(1, rev(dim(f))))

  with_logdir(temp, {
    log_event(hello = 1)
    log_event(hello = 2)
    log_event(hist = summary_histogram(rnorm(1000)))
    log_event(img = summary_image(img))
    log_event(audio = summary_audio(audio))
    log_event(text = summary_text("hello world"))
  })

  summaries <- collect_events(temp, type = "summary")
  expect_equal(value(summaries$summary[1]), 1)
  expect_error(value(summaries$summary), regexp = "single summary_value")

  histo <- value(summaries$summary[3])
  expect_true(is.data.frame(histo))
  expect_true(nrow(histo) == 30)
  expect_equal(names(histo), c("lower", "upper", "count"))

  im <- value(summaries$summary[4])
  expect_equal(class(im), c("matrix", "array"))
  expect_equal(dim(im), c(28, 28))

  aud <- value(summaries$summary[5])
  expect_equal(class(aud), c("matrix", "array"))
  expect_equal(dim(aud), c(2, 352800))
  expect_equal(attr(aud, "sample_rate"), 44100)
  expect_equal(attr(aud, "bit_depth"), 32)

  tx <- value(summaries$summary[6])
  expect_equal(tx, "hello world")

})

test_that("collect_event with type fails", {

  temp <- tempfile()
  with_logdir(temp, {
    log_event(hello = summary_histogram(runif(1000)))
    log_event(bye = 1)
  })

  expect_error(
    collect_events(temp, type = "xx"),
    regexp = "must be one of"
  )

  expect_error(
    collect_events(temp, n =-102),
    regexp = "must be positive"
  )

})

test_that("can format a data frame containing summaries", {


  temp <- tempfile()
  with_logdir(temp, {
    log_event(hello = summary_histogram(runif(1000)))
    log_event(bye = 1)
  })

  x <- collect_events(temp)
  expect_error(as.data.frame(x), regexp = NA)
  expect_error(format(as.data.frame(x)), regexp = NA)

})

test_that("can as list summary values", {

  temp <- tempfile()
  with_logdir(temp, {
    log_event(hello = runif(1))
    log_event(bye = 1)
  })

  expect_error(
    value(collect_events(temp, type = "summary")$summary),
    "as_list = TRUE"
  )

  x <- value(collect_events(temp, type = "summary")$summary, as_list = TRUE)
  expect_true(is.list(x))
  expect_length(x, 2)

})

#' Logs hyperparameters configuration
#'
#' Logs the hyperaparameter configuration for a HyperParameter tuning experiment.
#' It allows you to define the domain for each hyperparameters and what are the
#' metrics that should be shown in the TensorBoard UI, along with configuring
#' their display name and descriptions.
#'
#' @section Recommendations:
#'
#' When loging hyperparameter tuning experiments, the log directory organization
#' is:
#'
#' ```
#' - root:
#'  - log_hparams_config(...)
#'  - run1:
#'    - log_hparams(...)
#'    - log_event(...)
#'  - run2:
#'    - log_hparams(...)
#'    - log_event(...)
#' ```
#' Ie you should have a root logdir that will only contain the hyperaparameter
#' config log, as created with [log_hparams_config()]. Then each run in the
#' experiment will have it's own logdir as a child directory of the root logdir.
#'
#' @param hparams A list of `hparams` objects as created by [hparams_hparam()].
#' @param metrics A list of `metrics` objects as created by [hparams_metric()].
#'   These metrics will be tracked by TensorBoard UI when displaying the
#'   hyperparameter tuning table.
#' @inheritParams log_hparams
#'
#' @seealso [log_hparams()]
#' @returns Invisibly returns the HParam conffuration data as a `summary` object.
#'
#' @export
log_hparams_config <- function(hparams, metrics, time_created_secs = get_wall_time()) {
  log_event(
    summary_hparams_config(hparams, metrics, time_created_secs),
    step = 0
  )
}

#' @describeIn log_hparams_config For advanced users only. Creates a hyperaparameter
#'   configuration summary that can be logged with [log_event()].
#' @export
summary_hparams_config <- function(hparams, metrics, time_created_secs = get_wall_time()) {
  summary_values(
    summary_metadata(
      plugin_name = "hparams",
      plugin_content = new_hparams_hparams_plugin_data(
        version = 0,
        experiment = as_hparams_experiment(hparams, metrics, time_created_secs),
        session_start_info = NA
      )
    ),
    tag = "_hparams_/experiment", # this tag is recognized by tensorboard, can't change
    class = "summary_hparams_config"
  )
}

as_hparams_experiment <- function(hparams, metrics, time_created_secs) {
  hparams <- lapply(hparams, function(hparam) {
    info <- list(
      name = hparam$name,
      display_name = hparam$display_name,
      description = hparam$description
    )

    if (!rlang::is_named(hparam$domain)) {
      info$domain_discrete <- hparam$domain
    } else {
      info$domain_interval <- new_hparams_interval(
        min_value = hparam$domain["min_value"],
        max_value = hparam$domain["max_value"]
      )
    }

    do.call(hparams_hparam_info, info)
  })

  metrics <- lapply(metrics, function(metric) {
    new_hparams_metric_info(
      name = new_hparams_metric_name(group = metric$group, tag = metric$tag),
      display_name = metric$display_name,
      description = metric$description,
      dataset_type = metric$dataset_type
    )
  })

  new_hparams_experiment(
    name = NA,
    description = NA,
    user = NA,
    time_created_secs = time_created_secs,
    hparam_infos = list(vec_c(!!!hparams)),
    metric_infos = list(vec_c(!!!metrics))
  )
}

#' Defines a HParam
#'
#' Hparam object are used to describe names and domains of hyperparameters so
#' TensorBoard UI can show additional information about them.
#'
#' @param name Name of the hyperparameter.
#' @param domain A list of values that can be assumed by the hyperparameter.
#'   It can be `character()`, `numeric()` or `logical()` vector. You can also
#'   pass a named numeric vector with eg `c(min_value = 0, max_value = 10)` in
#'   this case, any value in this range is accepted.
#' @param display_name Display name of the hparameter for the TensorBoard UI.
#'   By default it's identical to the name.
#' @param description Parameter description. Shown in tooltips around the
#'   TensorBoard UI.
#'
#' @note A list of `hparam` values can be passed to [log_hparams_config()] so
#'   you define the hyperparameters that are tracked by the experiment.
#'
#' @returns A `hparams_hparam` object.
#' @examples
#' hparams_hparam("optimizer", domain = c("adam", "sgd"))
#' hparams_hparam("num_units", domain = c(128, 512, 1024))
#' hparams_hparam("use_cnn", domain = c(TRUE, FALSE))
#' hparams_hparam("dropout", domain = c(min_value = 0, max_value = 0.5))
#' @export
hparams_hparam <- function (name, domain = NA, display_name = name, description = name) {
  structure(list(
    name = name,
    domain = domain,
    display_name = display_name,
    description = description
  ), class = "hparams_hparam")
}

#' Defines a Metric
#'
#' Metric objects are passed to [log_hparams_config()] in order to define the
#' collection of scalars that will be displayed in the HParams tab in TensorBoard.
#'
#' @param tag The tag name of the scalar summary that corresponds to this
#'  metric.
#' @param group An optional string listing the subdirectory under the
#'   session's log directory containing summaries for this metric.
#'   For instance, if summaries for training runs are written to
#'   events files in `ROOT_LOGDIR/SESSION_ID/train`, then `group`
#'   should be `"train"`. Defaults to the empty string: i.e.,
#'   summaries are expected to be written to the session logdir.
#' @param display_name An optional human-readable display name.
#' @param description An optional Markdown string with a human-readable
#'   description of this metric, to appear in TensorBoard.
#' @param dataset_type dataset_type: Either `"training"` or `"validation`, or
#'   `NA`.
#'
#' @returns A `hparams_metric` object.
#' @examples
#' hparams_metric("loss", group = "train")
#' hparams_metric("acc")
#' @export
hparams_metric <- function(tag, group = NA,
                           display_name = tag,
                           description = tag,
                           dataset_type = NA) {

  if (is.na(dataset_type)) {
    dataset_type <- "unknown"
  }

  structure(list(
    tag = tag, group = group,
    display_name = display_name,
    description = description,
    dataset_type = dataset_type
  ), class = "hparams_metric")
}

#' Log hyperaparameters
#'
#' @param ... Named values of hyperparameters.
#' @param trial_id A name for the current trail. by default it's the hash of the
#'   hparams names and values.
#' @param time_created_secs The time the experiment is created in seconds
#'   since the UNIX epoch.
#'
#' @details This function should only be called once in a logdir and it will
#' record the set of hyperparameters used in that run. Undefined behavior can
#' happen if it's called more than once in a logdir - specially how TensorBoard
#' behaves during visualization.
#'
#' @returns A hyperparameter summary. USed for the side effects of logging the
#'   hyperparameter to the logdir.
#'
#' @seealso [log_hparams_config()]
#'
#' @examples
#' temp <- tempfile()
#' with_logdir(temp, {
#'   log_hparams(optimizer = "adam", num_units = 16)
#' })
#' @export
log_hparams <- function(..., trial_id = NA, time_created_secs = get_wall_time()) {
  log_event(summary_hparams(..., trial_id = trial_id, time_created_secs = time_created_secs), step = 0)
}

#' @describeIn log_hparams For advanced users only. It's recommended to use the `log_hparams()`
#' function instead. Creates a hyperparameter summary that can be written with `log_event()`.
#'
#' @export
summary_hparams <- function(..., trial_id = NA, time_created_secs = get_wall_time()) {
  hparams <- rlang::dots_list(..., .homonyms = "error")
  stopifnot(rlang::is_named(hparams))
  if (is.na(trial_id)) {
    trial_id <- digest::digest(hparams, algo = "sha256")
  }
  summary_values(
    metadata = summary_metadata(
      plugin_name = "hparams",
      plugin_content = new_hparams_hparams_plugin_data(
        version = 0,
        session_start_info = new_hparams_session_start_info(
          group_name = trial_id,
          start_time_secs = time_created_secs,
          hparams = list(hparams),
          model_uri = NA,
          monitor_url = NA
        ),
        experiment = NA
      )
    ),
    tag = "_hparams_/session_start_info", # this tag is read by TensorBoard, don't change.
    class = "summary_hparams_config"
  )
}

new_hparams_hparams_plugin_data <- function(version = integer(),
                                            experiment = new_hparams_experiment(),
                                            session_start_info = new_hparams_session_start_info()) {
  new_rcrd(
    fields = list(
      version = version,
      experiment = vec_cast(experiment, new_hparams_experiment()),
      session_start_info = vec_cast(session_start_info, new_hparams_session_start_info())
    ),
    class = "hparams_hparams_plugin_data"
  )
}

new_hparams_session_start_info <- function(group_name = character(),
                                           start_time_secs = integer(),
                                           hparams = list(),
                                           model_uri = character(),
                                           monitor_url = character()) {
  new_rcrd(
    fields = list(
      group_name = vec_cast(group_name, character()),
      start_time_secs = vec_cast(start_time_secs, integer()),
      hparams = vec_cast(hparams, list()),
      model_uri = vec_cast(model_uri, character()),
      monitor_url = vec_cast(monitor_url, character())
    ),
    class = "hparams_session_start_info"
  )
}

new_hparams_experiment <- function(name = character(), description = character(),
                           user = character(), time_created_secs = integer(),
                           hparam_infos = new_list_of(ptype=new_hparams_hparam_info()),
                           metric_infos = new_list_of(ptype=new_hparams_metric_info())) {
  new_rcrd(
    fields = list(
      name = vec_cast(name, character()),
      description = vec_cast(description, character()),
      user = vec_cast(user, character()),
      time_created_secs = vec_cast(time_created_secs, integer()),
      hparam_infos = hparam_infos,
      metric_infos = metric_infos
    ),
    class = "hparams_experiment"
  )
}

new_hparams_metric_info <- function(name = new_hparams_metric_name(),
                                    display_name = character(),
                                    description = character(),
                                    dataset_type = character()) {
  new_rcrd(
    fields = list(
      name = name,
      display_name = vec_cast(display_name, character()),
      description = vec_cast(description, character()),
      dataset_type = vec_cast(dataset_type, character())
    ),
    class = "hparams_metric_info"
  )
}

new_hparams_metric_name <- function(group = character(), tag = character()) {
  new_rcrd(
    fields = list(
      group = vec_cast(group, character()),
      tag = vec_cast(tag, character())
    ),
    class = "hparams_metric_name"
  )
}

hparams_hparam_info <- function(name, display_name = rep("", length(name)),
                                description = rep("", length(name)),
                                type = NA, ..., domain_discrete = NA,
                                domain_interval = NA) {

  stopifnot(!is.na(name))
  if (is.na(type)) {
    if (!is.na(domain_interval)) {
      type <- "float64"
    } else if (!rlang::is_na(domain_discrete)) {
      if (is.character(domain_discrete)) {
        type <- "string"
      } else if (is.numeric(domain_discrete)) {
        type <- "float64"
      } else if (is.logical(domain_discrete)) {
        type <- "bool"
      }
    }
  }

  if (!rlang::is_na(domain_discrete)) {
    domain_discrete <- list(domain_discrete)
  }

  new_hparams_hparam_info(
    name = vec_cast(name, character()),
    display_name = vec_cast(display_name, character()),
    description = vec_cast(description, character()),
    type = vec_cast(type, character()),
    domain_discrete = vec_cast(domain_discrete, list()),
    domain_interval = vec_cast(domain_interval, new_hparams_interval())
  )
}

new_hparams_hparam_info <- function(name = character(), display_name = character(),
                                    description = character(), type = character(),
                                    ...,
                                    domain_discrete = list(),
                                    domain_interval = new_hparams_interval()) {
  new_rcrd(
    fields = list(
      name = name,
      display_name = display_name,
      description = description,
      type = type,
      domain_discrete = domain_discrete,
      domain_interval = domain_interval
    ),
    class = "hparams_hparam_info"
  )
}

new_hparams_interval <- function(min_value = numeric(), max_value = numeric()) {
  new_rcrd(
    fields = list(
      min_value = min_value,
      max_value = max_value
    ),
    class = "hparams_interval"
  )
}

#' @export
vec_ptype2.hparams_session_start_info.hparams_session_start_info <- function(x, y, ...) {
  new_hparams_session_start_info()
}
#' @export
vec_cast.hparams_session_start_info.hparams_session_start_info <- function(x, to, ...) {
  x
}

#' @export
vec_ptype2.hparams_hparams_plugin_data.list <- function(x, y, ...) {
  list()
}
#' @export
vec_cast.list.hparams_hparams_plugin_data <- function(x, to, ...) {
  if (is.na(x))
    return(NA)

  if (vec_size(x) == 1)
    return(list(x))
}

#' @export
vec_ptype2.hparams_experiment.hparams_experiment <- function(x, y, ...) {
  new_hparams_session_start_info()
}
#' @export
vec_cast.hparams_experiment.hparams_experiment <- function(x, to, ...) {
  x
}

#' @export
vec_ptype2.tfevents_summary_values.summary_hparams_config <- function(x, y, ...) {
  new_summary_values()
}
#' @export
vec_ptype2.summary_hparams_config.tfevents_summary_values <- function(x, y, ...) {
  new_summary_values()
}

#' @export
vec_ptype2.hparams_interval.hparams_interval <- function(x, y, ...) {
  new_hparams_interval()
}
#' @export
vec_cast.hparams_interval.hparams_interval <- function(x, to, ...) {
  x
}

#' @export
vec_ptype2.hparams_hparam_info.hparams_hparam_info <- function(x, y, ...) {
  new_hparams_hparam_info()
}
#' @export
vec_cast.hparams_hparam_info.hparams_hparam_info <- function(x, to, ...) {
  x
}

skip_if_tbparse_not_available()

test_that("simple hparams experiment", {

  hparams <- list(
    hparams_hparam("dropout", domain = c(min_value = 0.1, max_value = 0.5)),
    hparams_hparam("optimizer", domain = c("adam", "sgd")),
    hparams_hparam("use_cnn", domain = c(TRUE, FALSE)),
    hparams_hparam("num_units", c(8, 12, 16))
  )

  metrics <- list(
    hparams_metric(tag = "loss"),
    hparams_metric(tag = "accuracy", group = "valid"),
    hparams_metric(tag = "f1", dataset_type = "training"),
    hparams_metric(tag = "f12", dataset_type = "validation")
  )

  temp <- tempfile()
  with_logdir(temp, {
    log_hparams_config(hparams, metrics)
    log_hparams(
      dropout = 0.1,
      optimizer = "adam",
      use_cnn = TRUE,
      num_units = 8
    )

    for (i in 1:10) {
      log_event(loss = runif(1), valid = list(accuracy = runif(1)),
                f1 = runif(1), f12 = runif(1))
    }
  })

  reader <- tbparse$SummaryReader(temp)

  expect_equal(reader$hparams$tag, c("dropout", "num_units", "optimizer", "use_cnn"))
  expect_equal(reader$hparams$value, list(0.1, 8, "adam", TRUE))
  expect_equal(nrow(reader$scalars), 40)

  expect_equal(nrow(collect_events(temp)), 40 + 4)
  expect_equal(nrow(collect_events(temp, type="summary")), 40 + 2)
  expect_equal(nrow(collect_events(temp, type="scalar")), 40)
})

test_that("multiple runs, each in a different logdir", {

  hparams <- list(
    hparams_hparam("dropout", domain = c(min_value = 0.1, max_value = 0.5)),
    hparams_hparam("optimizer", domain = c("adam", "sgd")),
    hparams_hparam("use_cnn", domain = c(TRUE, FALSE)),
    hparams_hparam("num_units", c(8, 12, 16))
  )

  metrics <- list(
    hparams_metric(tag = "loss"),
    hparams_metric(tag = "accuracy", group = "valid"),
    hparams_metric(tag = "f1", dataset_type = "training")
  )

  temp <- tempfile()

  with_logdir(temp, {
    log_hparams_config(hparams, metrics)
  })

  for(run in 1:10) {
    with_logdir(file.path(temp, paste0("run", run)), {
      log_hparams(
        dropout = runif(1, min = 0.1, max = 0.5),
        optimizer = sample(c("adam", "sgd"), 1),
        use_cnn = sample(c(TRUE, FALSE), 1),
        num_units = sample(c(8, 12, 16), 1)
      )

      for (i in 1:10) {
        log_event(loss = runif(1), valid = list(accuracy = runif(1)),
                  f1 = runif(1))
      }
    })
  }

  reader <- tbparse$SummaryReader(temp)
  expect_equal(nrow(reader$hparams), 40)
  expect_equal(nrow(collect_events(file.path(temp, "run1"))), 30 + 3)
})

test_that("write only hparams without the config", {

  temp <- tempfile()
  for(run in 1:10) {
    with_logdir(file.path(temp, paste0("run", run)), {
      log_hparams(
        dropout = runif(1, min = 0.1, max = 0.5),
        optimizer = sample(c("adam", "sgd"), 1),
        use_cnn = sample(c(TRUE, FALSE), 1),
        num_units = sample(c(8, 12, 16), 1)
      )

      for (i in 1:10) {
        log_event(loss = runif(1), valid = list(accuracy = runif(1)),
                  f1 = runif(1))
      }
    })
  }

  reader <- tbparse$SummaryReader(temp)
  expect_equal(nrow(reader$hparams), 40)

})

#' Creates a text summary
#' @param txt An object that can be converted to a text.
#' @param ... Currently unused.
#' @inheritParams summary_scalar
#' @family summary
#'
#' @examples
#' temp <- tempfile()
#' with_logdir(temp, {
#'   log_event(
#'     x = "hello world",
#'     y = summary_text("hello world")
#'   )
#' })
#' @returns A summary that can be logged with [log_event()].
#' @export
summary_text <- function(txt, ..., metadata = NULL, tag = NA) {
  UseMethod("summary_text")
}

#' @describeIn summary_text Creates a summary from a scalar character vector.
#' @export
summary_text.character <- function(txt, ..., metadata = NULL, tag = NA) {
  if (is.null(metadata)) {
    metadata <- summary_metadata(plugin_name = "text")
  }

  if (!rlang::is_scalar_atomic(txt)) {
    cli::cli_abort(c(
      "Can't log a character vector with length != 1.",
      i = "Expected a single value but got a vector with length {.val {length(txt)}}."
    ))
  }

  if (!all(field(metadata, "plugin_name") == "text")) {
    cli::cli_abort(c(
      "Plugin name should be 'text'",
      x = "Got {.val {unique(field(metadata, 'plugin_name'))}}"
    ))
  }

  summary_tensor(
    txt,
    dtype = "string",
    metadata = metadata,
    tag = tag
  )
}

test_that("can log text values", {
  temp <- tempfile()
  with_logdir(temp, {
    log_event(x = summary_text("Hello world!"))
    log_event(x = "## hello world!\n hello!")
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(log_path = temp)

  expect_equal(nrow(reader$text), 2)
})

test_that("errors when passing more than one character value", {
  temp <- tempfile()
  expect_error(
    with_logdir(temp, {
      log_event(x = c("hello", "world"))
    }),
    regexp = "Can't log a character"
  )
  expect_error(
    with_logdir(temp, {
      log_event(x = summary_text(c("hello", "world")))
    }),
    regexp = "Can't log a character"
  )
})

test_that("make sure we fail when metadata is not text", {
  temp <- tempfile()
  expect_error(
    with_logdir(temp, {
      log_event(x = summary_text("hello", metadata = summary_metadata("text2")))
    }),
    regexp = "Plugin name should be "
  )
})

#' Creates a image summary
#'
#' @param img An object that can be converted to an image.
#' @param ... Currently unused.
#' @inheritParams summary_scalar
#' @param width Width of the image.
#' @param height Height of the image.
#' @param colorspace Valid colorspace values are
#'   `1 - grayscale`,
#'   `2 - grayscale + alpha`,
#'   `3 - RGB`,
#'   `4 - RGBA`,
#'   `5 - DIGITAL_YUV`,
#'   `6 - BGRA`
#' @family summary
#' @returns An image summary that can be logged with [log_event()].
#' @examples
#' tmp <- tempfile()
#' with_logdir(tmp, {
#'   summary_image(array(runif(100), dim = c(1,10, 10, 1)))
#' })
#' @export
summary_image <- function(img, ..., metadata = NULL, tag = NA) {
  UseMethod("summary_image")
}

#' @describeIn summary_image Cretes an image summary from a ggplot2 graph object.
#'   The `...` will be forwarded to [grDevices::png()].
#' @export
summary_image.ggplot <- function(img, ..., width = 480, height = 480, metadata = NULL, tag = NA) {
  temp <- tempfile(fileext = ".png")
  on.exit({unlink(temp)}, add = TRUE)

  grDevices::png(filename = temp, width = width, height = height, ...)
  plot(img)
  grDevices::dev.off()

  sze <- fs::file_info(temp)$size
  raw <- readBin(temp, n = sze, what = "raw")

  summary_image(
    raw,
    width = width,
    height = height,
    colorspace = 4,
    metadata = metadata,
    tag = tag
  )
}

#' @describeIn summary_image Creates an image from an R array. The array should be
#'   numeric, with values between 0 and 1. Dimensions should be `(batch, height, width, channels)`.
#' @export
summary_image.array <- function(img, ..., metadata = NULL, tag = NA) {
  if (length(dim(img)) <= 3) {
    cli::cli_abort(c(
      "Expected an array with dimensions {.code (batch, height, width, channels)}",
      i = "Got an array with dimensions {.code ({paste(dim(img), collapse=', ')})}."
    ))
  }

  if (is.null(metadata)) {
    metadata <- summary_metadata(plugin_name = "images")
  }

  if (!all(field(metadata, "plugin_name") == "images")) {
    cli::cli_abort(c(
      "Plugin name should be 'images'",
      x = "Got {.val {unique(field(metadata, 'plugin_name'))}}"
    ))
  }

  # See https://github.com/tensorflow/tensorboard/blob/a74c10dd197e7b2a07219855a61bc62651e80065/tensorboard/plugins/image/summary_v2.py#L111
  # for the implementation.
  # The images are converted to a character evctor, the first 2 elements being the
  # dimensions, and the others containing the image encoded a png.
  png_images <- apply(img, 1, function(x) {
     png::writePNG(x)
  }, simplify = FALSE)
  blob_images <- blob::new_blob(png_images)

  dims <- dim(img)
  dims <- blob::blob(as.raw(dims[3]), as.raw(dims[2]))

  blobs <- c(dims, blob_images)

  summary_tensor(
    blobs,
    dtype = "string",
    metadata = metadata,
    tag = tag
  )
}

#' @describeIn summary_image Creates an image from [blob::blob()] vctr of PNG encoded images,
#'   (eg using [png::writePNG()]). `width`, `height` and `colorspace` are recycled
#'   thus they can be a single scalar or a vector the same size of the images blob.
#' @export
summary_image.blob <- function(img, ..., width, height, colorspace, metadata = NULL, tag = NA) {
  c(img, width, height, colorspace) %<-% vec_recycle_common(img, width, height, colorspace)

  image <- summary_summary_image(
    buffer = img,
    width = width,
    height = height,
    colorspace = colorspace
  )
  new_summary_image(image, metadata = metadata, tag = tag)
}

#' @describeIn summary_image Creates an image from a png encoded image. Eg, created
#'   with [png::writePNG()]. In this case you need to provide `width`, `height` and
#'   `colorspace` arguments.
#' @export
summary_image.raw <- function(img, ..., width, height, colorspace, metadata = NULL, tag = NA) {
  summary_image(
    img = blob::blob(img),
    width = width,
    height = height,
    colorspace = colorspace,
    metadata = metadata,
    tag = tag
  )
}

new_summary_image <- function(img = new_summary_summary_image(), ..., metadata = NULL, tag = character()) {
  if (is.null(metadata)) {
    metadata <- summary_metadata(plugin_name = "images")
  }
  summary_values(metadata = metadata, image = img, class = "tfevents_summary_image", tag = tag)
}

summary_summary_image <- function(buffer, width, height, colorspace) {
  new_summary_summary_image(
    buffer = vec_cast(buffer, blob()),
    width = width,
    height = height,
    colorspace = colorspace
  )
}

#' @importFrom blob blob
new_summary_summary_image <- function(buffer = blob(), width = integer(), height = integer(), colorspace = integer()) {
  buffer <- vec_cast(buffer, blob())
  vctrs::new_rcrd(
    fields = list(
      buffer = buffer,
      width = width,
      height = height,
      colorspace = colorspace
    ),
    class = "summary_summary_image"
  )
}

#' @export
vec_ptype2.summary_summary_image.summary_summary_image <- function(x, y, ...) {
  new_summary_summary_image()
}
#' @export
vec_cast.summary_summary_image.summary_summary_image <- function(x, to, ...) {
  x
}

#' Log event
#'
#' @param ... Named values that you want to log. They can be possibly nested,
#'  in this case, the enclosing names are considered 'run' names by TensorBoard.
#' @param step The step associated the logs. If `NULL`, a managed step counter
#'  will be used, and the global step is increased in every call to [log_event()].
#'
#' @note [log_event()] writes events to the default `logdir`. You can query the
#'  default `logdir` with [get_default_logdir()] and modify it with
#'  [set_default_logdir()]. You can also use the [with_logdir()] context switcher
#'  to temporarily modify the logdir.
#'
#' @return Invisibly returns the logged data.
#'
#' @examples
#' temp <- tempfile()
#' with_logdir(temp, {
#'   log_event(
#'      train = list(loss = runif(1), acc = runif(1)),
#'      valid = list(loss = runif(1), acc = runif(1))
#'   )
#' })
#' @export
log_event <- function(..., step = get_global_step(increment = TRUE)) {
  data <- rlang::dots_list(..., .named = FALSE, .homonyms = "error")
  write_event(as_event(data, step = step, wall_time = get_wall_time()))
  invisible(data)
}

write_event <- function(event) {
  writers <- lapply(fs::path(get_default_logdir(), field(event, "run")), get_writer)
  write_events(event, writers)
}

.tfevents <- new.env()

#' Query and modify the logdir
#'
#' [log_event()] has a notion of default logdir, so you don't need to specify it
#' at every call. These functions allow you to query and the current logdir.
#'
#' @param logdir The `logdir` that you want to set as default.
#' @param code Expressions that will be evaluated in a context with the `new`
#'   `logdir` as the default `logdir`.
#' @param .env Environment that controls scope of changes. For expert use only.
#'
#' @returns The `logdir` for `get_default_logdir()` otherwise invisibly returns
#'   `NULL`
#'
#' @examples
#' temp <- tempfile()
#' get_default_logdir()
#' with_logdir(temp, {
#'  print(get_default_logdir())
#' })
#' @export
get_default_logdir <- function() {
  if (is.null(.tfevents$logdir))
    set_default_logdir()
  rlang::env_get(.tfevents, "logdir")
}
#' @describeIn get_default_logdir Modifies the default `logdir`.
#' @export
set_default_logdir <- function(logdir = "logs") {
  rlang::env_bind(.tfevents, logdir = path.expand(logdir))
}

with_logdir_impl <- function() {
  with_logdir_ <- withr::with_(
    set = set_default_logdir,
    get = function(logdir) {
      get_default_logdir()
    }
  )
  function(logdir, code) {
    with_logdir_(logdir, code)
  }
}

#' @describeIn get_default_logdir Temporarily modify the default `logdir`.
#' @export
with_logdir <- with_logdir_impl()

#' @describeIn get_default_logdir Temporarily modify thedefault `logdir`.
#' @export
local_logdir <- function(logdir, .env = parent.frame()) {
  old <- get_default_logdir()
  set_default_logdir(logdir)
  withr::defer(set_default_logdir(old), envir = .env)
}

#' Global step counters
#'
#' @param step New value for `step`.
#' @param increment Wether to increment the `step` when getting it.
#'
#' @details `tfevents` tracks and automatically increased the step counter whenever
#' [log_event()] is called. Note that, it maintains a separate step counter for
#' each root `logdir`, thus if you change the `logdir` using [set_default_logdir()]
#' or [with_logdir()], a different step counter will be used.
#'
#' @returns The global step value for the default logdir, when `get_global_step`,
#'  otherwise returns `NULL` invisibly.
#'
#' @examples
#' temp <- tempfile()
#' with_logdir(temp, {
#'   print(get_global_step())
#'   set_global_step(100)
#'   print(get_global_step())
#' })
#' print(get_global_step())
#' @export
get_global_step <- function(increment = TRUE) {
  # a separate global step count is tracked for each root logdir.
  # the global step is queried once when calling `log_event`.
  logdir <- get_default_logdir()
  if (!rlang::env_has(.steps, logdir))
    set_global_step(-1)
  cur_step <- rlang::env_get(.steps, logdir) + as.integer(increment)
  set_global_step(cur_step)
  cur_step
}
#' @describeIn get_global_step Set the global step.
#' @export
set_global_step <- function(step) {
  rlang::env_poke(.steps, get_default_logdir(), as.integer(step), create = TRUE)
}
.steps <- new.env()

.writers <- new.env()
get_writer <- function(logdir = get_default_logdir()) {
  logdir <- fs::path_norm(logdir)
  writer <- rlang::env_get(.writers, nm = logdir, default = NULL)
  if (!is.null(writer))
    return(writer)

  writer <- create_event_writer(logdir)
  rlang::env_poke(
    .writers,
    logdir,
    writer,
    create = TRUE
  )
  writer
}

test_that("can write and visualize histograms", {

  temp <- tempfile()
  with_logdir(temp, {
    for(i in 1:10) {
      log_event(x = summary_histogram(rnorm(10000)))
    }
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)
  x <- reader$tensors
  expect_equal(nrow(x), 10)

  # test for arrays.
  temp <- tempfile()
  with_logdir(temp, {
    for(i in 1:10) {
      log_event(x = summary_histogram(array(rnorm(10000), dim = c(10, 10, 100))))
    }
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)
  x <- reader$tensors
  expect_equal(nrow(x), 10)
})

test_that("edge case where there's no data doesn't fail", {
  temp <- tempfile()
  with_logdir(temp, {
    for(i in 1:10) {
      log_event(x = summary_histogram(numeric(0)))
    }
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)
  x <- reader$tensors
  expect_equal(nrow(x), 10)

  temp <- tempfile()
  with_logdir(temp, {
    for(i in 1:10) {
      log_event(x = summary_histogram(rep(0, 100)))
    }
  })

  reader <- tbparse$SummaryReader(temp)
  x <- reader$tensors
  expect_equal(nrow(x), 10)
})

test_that("can write a tensor", {
  skip_if_tbparse_not_available()

  tx <- array(0, dim = c(28, 28, 28))
  txc <- array("hello", dim = c(28, 28, 28))
  temp <- tempfile()
  with_logdir(temp, {
    log_event(x = summary_tensor(tx, "float"))
    log_event(y = summary_tensor(tx)) # auto detect type
    log_event(z = summary_tensor(txc)) # auto detect type
    log_event(a = summary_tensor(list(tx, txc))) # auto detect type
    log_event(b = summary_tensor(tx, "double"))
  })

  reader <- tbparse$SummaryReader(temp)
  df <- reader$tensors
  expect_equal(nrow(df), 6)
})

test_that("can write tensors with dimnames", {
  tx <- array(0, dim = c(1, 2, 2, 3))
  names(dim(tx)) <- c("b", "w", "h", "c")

  temp <- tempfile()
  with_logdir(temp, {
    log_event(x = summary_tensor(tx))
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)

  df <- reader$tensors
  # TODO: can't easily test if dimension names we actually writen because
  # tbparse ignores them.
  expect_equal(dim(df$value[[1]]), c(1,2,2,3))
})

test_that("can set name within summary_tensor", {
  tx <- array(0, dim = c(1, 28, 28, 28))
  temp <- tempfile()
  with_logdir(temp, {
    log_event(summary_tensor(tx, tag = "hello"))
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)
  expect_equal(reader$tags$tensors, "hello")
})

test_that("can write an audio file", {
  skip_if_not_installed("wav")
  f <- wav::read_wav(test_path("resources/test-audio.wav"))
  audio <- array(t(f), dim = c(1, rev(dim(f))))

  temp <- tempfile()
  with_logdir(temp, {
    log_event(x = summary_audio(audio))
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)

  # couldn't find a way to decode the binary string directly from memory,
  # so we write to disk and read again
  temp2 <- tempfile()
  w <- reticulate::import_builtins()$open(temp2, "wb")
  w$write(reader$tensors$value[[1]][[1]])

  f2 <- wav::read_wav(temp2)
  expect_true(all.equal(as.numeric(f), as.numeric(f2)))
})

test_that("can write multiple audio files from a array", {
  skip_if_not_installed("wav")
  f <- wav::read_wav(test_path("resources/test-audio.wav"))
  f_t <- t(f)
  audio <- array(0, dim = c(10, rev(dim(f))))
  for (i in 1:10) {
    audio[i,,] <- f_t
  }

  temp <- tempfile()
  with_logdir(temp, {
    log_event(x = summary_audio(audio))
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)

  for (i in 1:10) {
    temp2 <- tempfile()
    w <- reticulate::import_builtins()$open(temp2, "wb")
    w$write(reader$tensors$value[[1]][[i]])

    f2 <- wav::read_wav(temp2)
    expect_true(all.equal(as.numeric(f), as.numeric(f2)))
  }
})

test_that("can write directly from raw encoded file", {
  path <- test_path("resources/test-audio.wav")
  audio <- readBin(path, what = raw(), n = fs::file_info(path)$size)

  temp <- tempfile()
  with_logdir(temp, {
    log_event(x = summary_audio(audio))
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)

  # couldn't find a way to decode the binary string directly from memory,
  # so we write to disk and read again
  temp2 <- tempfile()
  w <- reticulate::import_builtins()$open(temp2, "wb")
  w$write(reader$tensors$value[[1]][[1]])

  expect_true(all.equal(wav::read_wav(path), wav::read_wav(temp2)))
})

test_that("write a few simple scalars", {
  temp <- tempfile()
  with_logdir(temp, {
    for(i in 1:10) {
      log_event(hello = i^2)
    }
  })

  events <- collect_events(temp)
  expect_equal(nrow(events), 10 + 1)
  expect_equal(events$run, rep(".", 11))

  scalars <- collect_events(temp, type="scalar")
  expect_equal(nrow(events), 10 + 1)
  expect_equal(scalars$value, (1:10)^2)
  expect_equal(scalars$step, 0:9)
  expect_equal(scalars$tag, rep("hello", 10))
})

test_that("write nested scalar for multiple runs", {
  temp <- tempfile()
  with_logdir(temp, {
    for(i in 1:10) {
      log_event(
        train = list(loss = i, acc = i^2),
        valid = list(loss = i+1, acc = (i+1)^2)
      )
    }
  })

  events <- collect_events(temp)
  expect_equal(nrow(events), 4*10 + 2)
  expect_equal(unique(events$run), c("train", "valid"))

  scalars <- collect_events(temp, type = "scalar")
  expect_equal(nrow(scalars), 4*10)
  expect_equal(unique(scalars$tag), c("loss", "acc"))
  expect_true(all(unique(scalars$step) %in% 0:9))
})

test_that("can log manually created scalars directly", {

  temp <- tempfile()
  with_logdir(temp, {
    for(i in 1:10) {
      log_event(
        train = list(loss = i, acc = i^2),
        valid = list(loss = i+1, acc = (i+1)^2),
        test = list(loss2 = summary_scalar(i+2))
      )
    }
  })

  events <- collect_events(temp)
  expect_equal(nrow(events), 4*10 + 10 + 3)

  scalars <- collect_events(temp, type = "scalar")
  expect_equal(scalars[scalars$tag == "loss2",]$value, 1:10 + 2)
})

test_that("can log with a specified step", {

  temp <- tempfile()
  with_logdir(temp, {
    log_event(hello = 1)
    log_event(hello = 1)
    log_event(hello = 1)
    log_event(hello = 1, step = 100)
    log_event(hello = 1)
    log_event(bye = 1, step = get_global_step(increment = FALSE))
  })

  scalars <- collect_events(temp, type = "scalar")
  expect_true(100 %in% scalars$step)
  expect_true(3 %in% scalars$step)
  expect_equal(scalars$step[scalars$tag == "bye"], 3)
})

test_that("local_logdir", {
  temp1 <- tempfile()
  temp2 <- tempfile()
  f <- function() {
    local_logdir(temp2)
    get_default_logdir()
  }

  with_logdir(temp1, {
    expect_equal(temp1, get_default_logdir())
    expect_equal(temp2, f())
    expect_equal(temp1, get_default_logdir())
  })

})

test_that("can write tags with the slash instead of nested list", {
  temp <- tempfile()
  with_logdir(temp, {
    log_event(
      "train/loss" = 0.1,
      "train/acc" = 0.1,
      "valid/loss" = 0.1,
      "valid/acc" = 0.1
    )
  })

  expect_equal(nrow(collect_events(temp, type = "summary")), 4)
})

test_that("Errors gracefully when a numeric with length >1 is provided", {

  temp <- tempfile()
  expect_error({
    with_logdir(temp, {
      log_event(
        x = c(0, 1)
      )
    })
  }, regexp = "Can't log")

  expect_error({
    with_logdir(temp, {
      log_event(
        x = numeric()
      )
    })
  }, regexp = "Can't log")

})

map2 <- function(x, f, ...) {
  out <- vector(mode="list")
  nms <- names2(x)
  # because of unnamed object, and the possibility of duplicated names
  # we iterate via integer indexes and then re-add the names.
  for(i in seq_along(nms)) {
    out[[i]] <- f(x[[i]], nms[i], ...)
  }
  names(out) <- nms
  out
}

# Unnamed object will now have names="" for all obejcts.
# this is similar to the approach of `names(c(1, x = 2))`
# that makes the name for the first element an empty character.
names2 <- function(x) {
  if (is.null(names(x))) {
    return(rep("", length(x)))
  } else {
    names(x)
  }
}

#' Summary audio
#'
#' Audio summaries can be played withing the TensorBoard UI.
#'
#' @param audio Object that will be written as an audio event in the tfevents record.
#' @inheritParams summary_image
#' @param sample_rate The sample rate in Hz associated to the audio values.
#' @returns An audio summary that can be logged with [log_event()].
#' @family summary
#' @examplesIf rlang::is_installed("wav")
#' tmp <- tempfile()
#' with_logdir(tmp, {
#'   summary_audio(array(runif(100), dim = c(1,100, 1)))
#' })
#' @export
summary_audio <- function(audio, ..., metadata = NULL, tag = NA) {
  UseMethod("summary_audio")
}

#' @describeIn summary_audio Creates a summary from a 3D array with dimensions
#'   `(batch_size, n_samples, n_channels)`. Values must be in the range `[-1, 1]`.
#' @export
summary_audio.array <- function(audio, ..., sample_rate = 44100, metadata = NULL,
                          tag = NA) {

  rlang::check_installed("wav")
  temp <- tempfile()
  raw_audios <- apply(audio, 1, simplify = FALSE, function(x) {
    wav::write_wav(t(x), sample_rate = sample_rate, path = temp)
    sze <- fs::file_info(temp)$size
    readBin(temp, n = sze, what = "raw")
  })
  blob_audios <- blob::new_blob(raw_audios)
  summary_audio(
    blob_audios,
    metadata = metadata,
    tag = tag
  )
}

#' @describeIn summary_audio Creates an audio summary from a raw vector containing
#'  a WAV encoded audio file.
#' @export
summary_audio.raw <- function(audio, ..., metadata = NULL, tag = NA) {
  summary_audio(blob::blob(audio), metadata = metadata, tag = tag)
}

#' @describeIn summary_audio Creates an audio summary from a blob (ie list of raw vectors)
#'   containing WAV encoded audio files.
#' @export
summary_audio.blob <- function(audio, ..., metadata = NULL, tag = NA) {

  if (is.null(metadata)) {
    metadata <- summary_metadata(plugin_name = "audio")
  }

  summary_tensor(
    audio,
    dtype = "string",
    metadata = metadata,
    tag = tag
  )
}

summary_tensor <- function(x, dtype = NA, ..., metadata = NULL, tag = NA) {
  new_summary_tensor(x = x, dtype = dtype, metadata = metadata, tag = tag)
}

new_summary_tensor <- function(x, dtype = NA, ..., metadata = NULL, tag = NA) {
  if (is.null(metadata)) {
    metadata <- summary_metadata(plugin_name = "tensor")
  }
  summary_values(metadata = metadata, tensor = as_tensor_proto(x, dtype),
                 class = "tfevents_summary_tensor", tag = tag)
}

as_tensor_proto <- function(x, dtype = NA, ...) {
  if (rlang::is_na(x)) return(vec_cast(NA, new_tensor_proto()))
  UseMethod("as_tensor_proto")
}

#' @exportS3Method
as_tensor_proto.blob <- function(x, dtype = NA, ...) {
  if (is.na(dtype)) dtype <- "string"
  if (!dtype %in% c("string"))
    cli::cli_abort("dtype should be string when converting a blob to tensor proto.")
  tensor_proto(list(x), shape = new_tensor_shape(dim = length(x)), dtype = dtype)
}

#' @exportS3Method
as_tensor_proto.character <- function(x, dtype = NA, ...) {
  if (is.na(dtype)) dtype <- "string"
  if (!dtype %in% c("string"))
    cli::cli_abort("dtype should be string when converting a character to tensor proto.")
  tensor_proto(list(x), shape = new_tensor_shape(dim = length(x)), dtype = dtype)
}

#' @exportS3Method
as_tensor_proto.array <- function(x, dtype = NA, ...) {
  dims <- dim(x)
  # proto store tensor data in C ordering, thus we need to reshape values
  # here.
  x <- aperm(x, rev(seq_along(dims)))
  tensor_proto(x, shape = new_tensor_shape(dim = list(dims)), dtype = dtype)
}

#' @exportS3Method
as_tensor_proto.list <- function(x, dtype, ...) {
  c(x, dtype) %<-% vec_recycle_common(x, dtype)
  results <- lapply(seq_along(x), function(i) {
    as_tensor_proto(x[[i]], dtype[[i]], ...)
  })
  vec_c(!!!results)
}

tensor_proto <- function(content, shape, dtype = NA) {
  if (!is.list(content)) content <- list(content)

  if ((length(dtype) == 1) && is.na(dtype)) {
    dtype <- sapply(content, make_default_dtype)
  }

  if (!inherits(shape, "tensor_shape")) {
    if (is.list(shape))
      shape <- new_tensor_shape(dim = shape)
  }


  new_tensor_proto(
    content = content,
    shape = shape,
    dtype = dtype
  )
}

make_default_dtype <- function(x) {
  if (is.numeric(x)) {
    return("float")
  }
  if (is.character(x)) {
    return("string")
  }
}

new_tensor_proto <- function(content = new_list_of(), shape = new_tensor_shape(), dtype = character()) {
  new_rcrd(
    fields = list(
      content = content,
      shape = shape,
      dtype = dtype
    ),
    class = "tensor_proto"
  )
}

new_tensor_shape <- function(dim = new_list_of(ptype = integer())) {
  new_rcrd(
    fields = list(
      dim = dim
    ),
    class = "tensor_shape"
  )
}

#' @export
vec_cast.tensor_proto.tensor_proto <- function(x, to, ...) x

#' Creates an histogram summary
#'
#' Writes an histogram for later analysis in TensorBoard's Histograms and
#' Distributions tab.
#'
#' @param data A Tensor of any shape. The histogram is computed over its elements,
#'   which must be castable to float64.
#' @inheritParams summary_scalar
#' @param buckets Optional positive int. The output will have this many buckets,
#'   except in two edge cases. If there is no data, then there are no buckets.
#'   If there is data but all points have the same value, then all buckets' left
#'   and right endpoints are the same and only the last bucket has nonzero count.
#'   Defaults to 30 if not specified.
#'
#' @returns
#' An histogram summary that can be logged with [log_event()].
#' @family summary
#'
#' @examples
#' temp <- tempfile()
#' with_logdir(temp, {
#'   for(i in 1:10) {
#'     log_event(x = summary_histogram(rnorm(10000)))
#'   }
#' })
#'
#' @export
summary_histogram <- function(data, ..., metadata = NULL, tag = NA) {
  UseMethod("summary_histogram")
}

#' @describeIn summary_histogram Creates an histogram summary for a numeric vector.
#' @export
summary_histogram.numeric <- function(data, ..., metadata = NULL, tag = NA, buckets = 30) {
  # this is mostly a literal translation from code in:
  # https://github.com/tensorflow/tensorboard/blob/2cd515880ea26ec66cfa85fbb38ad96cc38f6985/tensorboard/plugins/histogram/summary_v2.py#L41
  if (buckets == 0 || length(data) == 0) {
    histogram_buckets <- array(0, dim = c(buckets, 3))
  } else {
    min_ <- min(data)
    max_ <- max(data)
    range_ <- max_ - min_
    if (range_ == 0) {
      left_edges <- right_edges <- rep(min_, buckets)
      bucket_counts <- c(rep(0, buckets-1), length(data))
      histogram_buckets <- cbind(left_edges, right_edges, bucket_counts)
    } else {
      bucket_width <- range_ / buckets
      offsets <- data - min_
      bucket_indices <- floor(offsets / bucket_width)
      clamped_indices <- pmin(bucket_indices, buckets)
      bucket_counts <- sapply(seq(0, buckets-1), function(i) sum(clamped_indices==i))
      edges <- seq(min_, max_, length.out = buckets + 1)
      left_edges <- utils::head(edges, buckets)
      right_edges <- utils::tail(edges, buckets)
      histogram_buckets <- cbind(left_edges, right_edges, bucket_counts)
    }
  }

  if (is.null(metadata)) {
    metadata <- summary_metadata(plugin_name = "histograms")
  }

  summary_tensor(
    histogram_buckets,
    dtype = "double",
    metadata = metadata,
    tag = tag
  )
}

#' @describeIn summary_histogram Creates an histogram for array data.
#' @export
summary_histogram.array <- function(data, ..., metadata = NULL, tag = NA, buckets = 30) {
  summary_histogram(
    as.numeric(data),
    ...,
    metadata = metadata,
    tag = tag,
    buckets = buckets
  )
}

create_event_writer <- function(logdir) {
  fs::dir_create(logdir, recurse = TRUE)
  fname <- file.path(
    logdir,
    paste0("events.out.tfevents.", as.integer(Sys.time()), ".v2")
  )
  fs::file_create(fname)
  event_writer(fname)
}

#' Coerce an object to a event.
#'
#' @param x Object that will be coerced to an event.
#' @param step The step that will be used when the event is logged. This is used
#'   by TensorBoard when showing data.
#' @param wall_time The all time the event will appended to the event. This field
#'   is used by TensorBoard when displaying information based on actual time.
#' @param ... currently unused.
#'
#' @returns A event vctr with class <tfevents_event>.
#'
#' @section Extending `as_event`:
#'
#' `as_event` is an S3 generic and you can implement method for your own class.
#' We don't export the `event` constructor though, so you should implement it
#' in terms of other `as_event` methods.
#'
#' @examples
#' as_event(list(hello = 1), step = 1, wall_time = 1)
#'
#' @export
as_event <- function(x, step, wall_time, ...) {
  UseMethod("as_event")
}

#' @export
as_event.list <- function(x, step, wall_time, ..., name = ".") {
  ev <- map2(
    x,
    function(obj, nm) {
      as_event(obj, step = step, wall_time = wall_time, name = c(name, nm))
    }
  )
  vec_c(!!!squash_if(unname(ev), vec_is_list))
}

squash_if <- function(x, predicate) {
  lapply(x, function(obj) {
    if (predicate(obj)) {
      unlist(obj)
    } else {
      obj
    }
  })
}

#' @export
as_event.numeric <- function(x, step, wall_time, ..., name) {
  if (!rlang::is_scalar_atomic(x)) {
    cli::cli_abort(c(
      "Can't log a numeric vector with length != 1.",
      i = "Expected a single value but got a vector with length {.val {length(x)}}."
    ))
  }
  x <- summary_scalar(x)
  as_event(x, step = step, wall_time = wall_time, name = name)
}

#' @export
as_event.character <- function(x, step, wall_time, ..., name) {
  x <- summary_text(x)
  as_event(x, step = step, wall_time = wall_time, name = name)
}

#' @importFrom utils tail
#' @export
as_event.tfevents_summary_values <- function(x, step, wall_time, ..., name) {
  field(x, "tag") <- make_tag(field(x, "tag"), tail(name, 1))
  event(
    run = paste0(name[-length(name)], collapse = "/"),
    wall_time = wall_time,
    step = step,
    summary = summary(list(x))
  )
}

make_tag <- function(cur_tag, name) {
  name <- rep(name, length(cur_tag))

  if (any(name[!is.na(cur_tag)] != "")) {
    cli::cli_abort(c(
      x = "Two tags were provided for the same summary.",
      i = "You can only a specify tags once for a summary."
    ))
  }

  cur_tag[is.na(cur_tag)] <- name[is.na(cur_tag)]

  if (any(cur_tag == "")) {
    cli::cli_abort(c(
      x = "All summaries must have a tag, but found at least one without one.",
      i = "See {.help log_event} to find out how to specify tags for summaries."
    ))
  }

  cur_tag
}

#' Creates events
#'
#' We try to match events as closely as possible to the protobuf messages.
#' The hierarchy looks like:
#' ```
#' event (<event>):
#'  - run (<character>)
#'  - wall_time (<integer>)
#'  - step (<integer>)
#'  - summary (<summary> aka list_of<summary_values>):
#'     - values (list):
#'       - <summary_value>:
#'         - metadata (<summary_metadata>)
#'         - tag (<character>)
#'         - value (<numeric>)
#'         - image (<summary_summary_image>)
#'           - buffer (<blob>)
#'           - width (<integer>)
#'           - height (<integer>)
#'           - colorspace (<integer>)
#' ```
#'
#' @keywords internal
#' @import vctrs
event <- function(run, wall_time, step, ..., summary = NA, file_version = NA) {
  new_event(
    run = vec_cast(run, character()),
    wall_time = as.integer(wall_time),
    step = as.integer(step),
    summary = vec_cast(summary, new_summary()),
    file_version = vec_cast(file_version, character())
  )
}

new_event <- function(run = character(),
                        wall_time = integer(),
                        step = integer(),
                        ...,
                        summary = new_summary(),
                        file_version = character()) {
  new_rcrd(list(
    run = run,
    wall_time = wall_time,
    step = step,
    summary = summary,
    file_version = file_version
  ), class = "tfevents_event")
}


#' @export
format.tfevents_event <- function(x, ...) {
  paste0("<", field(x, "run"),"/", format(field(x, "step")), ">")
}

#' @export
vec_ptype2.tfevents_event.tfevents_event <- function(x, y, ...) {
  x
}

#' @export
vec_cast.tfevents_event.tfevents_event <- function(x, to, ...) {
  x
}

summary_values <- function(metadata, tag = NA, ..., value = NA, image = NA, tensor = NA, class = NULL) {
  value <- vec_cast(value, numeric())
  image <- vec_cast(image, new_summary_summary_image())
  tag <- vec_cast(tag, character())
  tensor <- vec_cast(tensor, new_tensor_proto())

  c(metadata, tag, value, image, tensor) %<-% vec_recycle_common(metadata, tag, value, image, tensor)

  new_summary_values(metadata = metadata, tag = tag, value = value, image = image,
                     tensor = tensor, class = class)
}

new_summary_values <- function(metadata = new_summary_metadata(), tag = character(), ...,
                        value = numeric(), image = new_summary_summary_image(),
                        tensor = new_tensor_proto(), class = NULL) {
  vctrs::new_rcrd(
    fields = list(metadata = metadata, tag = tag, value = value, image = image,
                  tensor = tensor),
    class = c(class, "tfevents_summary_values")
  )
}

summary <- function(values) {
  new_summary(values)
}

new_summary <- function(values = list(new_summary_values())) {
  new_list_of(values, ptype=new_summary_values(), class = "tfevents_summary")
}

#' @export
vec_ptype2.tfevents_summary_values.tfevents_summary_values <- function(x, y, ...) {
  new_summary_values()
}

#' @export
vec_ptype2.tfevents_summary_values.tfevents_summary <- function(x, y, ...) {
  new_summary()
}
#' @export
vec_ptype2.tfevents_summary.tfevents_summary_values <- function(x, y, ...) {
  new_summary()
}

# vec_cast.vctrs_percent.double <- function(x, to, ...) percent(x)
# vec_cast.double.vctrs_percent <- function(x, to, ...) vec_data(x)

#' @export
vec_cast.tfevents_summary_values.tfevents_summary_values <- function(x, to, ...) {
  x
}
#' @export
vec_cast.tfevents_summary.tfevents_summary_values <- function(x, to, ...) {
  if (is.na(x)) return(vec_cast(NA, new_summary()))
  new_summary(list(x))
}


#' Summary metadata
#'
#' Creates a summary metadata that can be passed to multiple `summary_` functions.
#'
#' @param plugin_name The name of the TensorBoard plugin that might use the summary.
#' @param display_name Display name for the summary.
#' @param description A description of the summary.
#' @param plugin_content An optional plugin content. Note that it will only be
#'  used if the C++ function `make_plugin_data` is aware of `plugin_content`
#'  for the specified plugin name. For advanced use only.
#' @param ... Currently unused. For future expansion.
#'
#' @returns A `summary_metadata` object.
#'
#' @examples
#' summary <- summary_scalar(1, metadata = summary_metadata("scalars"))
#'
#' @export
summary_metadata <- function(
    plugin_name,
    display_name = NA_character_,
    description = NA_character_, ...,
    plugin_content = NA) {
  rlang::check_dots_empty()
  plugin_content <- vec_cast(plugin_content, list())
  new_summary_metadata(plugin_name = plugin_name, display_name = display_name,
                       description = description,
                       plugin_content = plugin_content)
}

new_summary_metadata <- function(plugin_name = character(), display_name = character(),
                                 description = character(), plugin_content = list()) {
  vctrs::new_rcrd(
    fields = list(
      plugin_name = plugin_name,
      display_name = display_name,
      description = description,
      plugin_content = plugin_content
    ),
    class = "tfevents_summary_metadata"
  )
}

#' @export
format.tfevents_summary <- function(x, ...) {
  sapply(x, format)
}

#' @export
format.tfevents_summary_values <- function(x, ...) {
  paste0("<", field(x, "tag"), ">")
}

# These values are used from the C++ code
vec_c_list <- function(x) {
  vec_c(!!!x)
}
na <- NA
is_na <- function(x) {
  if (inherits(x, "vctrs_vctr"))
    vec_any_missing(x)
  else if (is.null(x))
    TRUE
  else
    rlang::is_na(x)
}

#' Scalar event
#'
#' @param value A numeric scalar value to be logged.
#' @param ... Currently unused. To allow future expansion.
#' @param metadata A `metadata` object, as created with [summary_metadata()]. In
#'   most cases you don't need to change the default.
#' @param tag A tag that within the TensorBoard UI. See [log_event()] for other
#'   ways of specifying the tag attribute.
#'
#' @returns A `<scalar_event>` object.
#' @family summary
#'
#' @examples
#' temp <- tempfile()
#' with_logdir(temp, {
#'   log_event(loss = summary_scalar(1))
#' })
#' @export
summary_scalar <- function(value, ..., metadata = NULL, tag = NA) {
  rlang::check_dots_empty()
  new_summary_scalar(value, metadata = metadata, tag = tag)
}

new_summary_scalar <- function(value = numeric(), ..., metadata = NULL,
                               tag = character()) {
  if (is.null(metadata)) {
    metadata <- summary_metadata(plugin_name = "scalars")
  }
  summary_values(metadata = metadata, value = value, tag = tag,
                 class = "tfevents_summary_scalar")
}

#' @export
vec_ptype2.tfevents_summary_values.tfevents_summary_scalar <- function(x, y, ...) {
  new_summary_values()
}
#' @export
vec_ptype2.tfevents_summary_scalar.tfevents_summary_values <- function(x, y, ...) {
  new_summary_values()
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

create_event_file_iterator <- function(path, run_name) {
    .Call(`_tfevents_create_event_file_iterator`, path, run_name)
}

event_file_iterator_next <- function(iter) {
    .Call(`_tfevents_event_file_iterator_next`, iter)
}

get_wall_time <- function() {
    .Call(`_tfevents_get_wall_time`)
}

event_writer <- function(file) {
    .Call(`_tfevents_event_writer`, file)
}

write_events <- function(events, writers) {
    .Call(`_tfevents_write_events`, events, writers)
}

skip_if_tbparse_not_available <- function() {
  skip_if(inherits(try(reticulate::import("tbparse"), silent = TRUE), "try-error"))
}

skip_if_no_tensorflow <- function() {
  skip_on_cran()
  skip_if_not_installed("tensorflow")
  skip_if(inherits(try(reticulate::import("tensorflow"), silent = TRUE), "try-error"))
}

if (inherits(try(reticulate::import("tbparse"), silent = TRUE), "try-error")) {
  tbparse <- NULL
} else {
  tbparse <- reticulate::import("tbparse")
}

test_that("write image", {
  temp <- tempfile()
  orig_img <- png::readPNG(test_path("resources/img.png"))
  img <- array(orig_img, dim = c(1, 28, 28, 1))

  with_logdir(temp, {
    log_event(hello = summary_image(img))
  })

  events <- collect_events(temp)
  expect_equal(nrow(events), 2)


  skip_if_tbparse_not_available()
  skip_if_no_tensorflow()
  reader <- tbparse$SummaryReader(temp)
  buf <- reader$tensors$value[[1]][[3]]
  # TODO: in theory we don't need tensorflow for this, but couldnt find a way to
  # cast the bytestring to a raw vector.
  reloaded <- as.array(tensorflow::tf$image$decode_png(buf))

  expect_equal(orig_img, reloaded[,,1]/255)
})

test_that("can write nested images", {
  temp <- tempfile()
  orig_img <- png::readPNG(test_path("resources/img.png"))
  img <- array(orig_img, dim = c(1, 28, 28, 1))

  with_logdir(temp, {
    log_event(
      train = list(im = summary_image(img)),
      valid = list(im = summary_image(img))
    )
  })

  events <- collect_events(temp)
  expect_equal(nrow(events), 4)
})

test_that("can write a batch of images", {
  orig_img <- png::readPNG(test_path("resources/img.png"))
  img <- array(orig_img, dim = c(28, 28, 1))
  arr <- array(0, dim = c(10, 28, 28, 1))
  for(i in 1:10) {
    arr[i,,,] <- img
  }

  temp <- tempfile()
  with_logdir(temp, {
    log_event(hello = summary_image(arr))
  })

  skip_if_tbparse_not_available()
  reader <- tbparse$SummaryReader(temp)
  tags <- reader$tags
  expect_equal(as.character(tags$tensors), "hello")
})

test_that("can write a ggplot", {
  df <- data.frame(x = 1:10, y = 1:10)
  gg <- ggplot2::ggplot(df, ggplot2::aes(x, y)) +
    ggplot2::geom_point()
  temp <- tempfile()
  with_logdir(temp, {
    log_event(hello = summary_image(gg))
  })

  events <- collect_events(temp)
  expect_equal(nrow(events), 2)

  buf <- field(field(events$summary[[2]], "image"), "buffer")
  reloaded <- png::readPNG(as.raw(buf[[1]]))

  expect_equal(length(dim(reloaded)), 3)
})

test_that("error when passing an array with wrong dimensions", {
  orig_img <- png::readPNG(test_path("resources/img.png"))
  img <- array(orig_img, dim = c(28, 28, 1))
  temp <- tempfile()

  expect_error({
    with_logdir(temp, {
      log_event(hello = summary_image(img))
    })
  }, regexp = "array with dimensions")

})

test_that("can pass a custom display name", {

  temp <- tempfile()
  orig_img <- png::readPNG(test_path("resources/img.png"))
  img <- array(orig_img, dim = c(1, 28, 28, 1))

  with_logdir(temp, {
    log_event(hello = summary_image(
      img,
      metadata = summary_metadata(
        plugin_name = "images",
        display_name = "Hello world",
        description = "This is a description"
      )
    ))
  })

  s <- collect_events(temp, type="summary")
  expect_equal(
    field(field(s$summary, "metadata"), "description"),
    "This is a description"
  )
  expect_equal(
    field(field(s$summary, "metadata"), "display_name"),
    "Hello world"
  )
})

test_that("fails if weong metadata is specified", {
  orig_img <- png::readPNG(test_path("resources/img.png"))
  img <- array(orig_img, dim = c(1, 28, 28, 1))
  expect_error({
    summary_image(img, metadata = summary_metadata(plugin_name = "tensor"))
  }, regexp = "Plugin name should be")
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{value}
\alias{value}
\alias{value.tfevents_summary_values}
\title{Extracts the value of a summary value}
\usage{
value(x, ...)

\method{value}{tfevents_summary_values}(x, ..., as_list = FALSE)
}
\arguments{
\item{x}{A \code{tfevents_summary_values} object.}

\item{...}{Currently unused. To allow future extension.}

\item{as_list}{A boolean indicating if the results should be returned in a list.
The default is to return a single value. If you need to extract values from
multiple summaries use \code{as_list = TRUE}.}
}
\value{
Depending on the type of the summary it returns an image, audio, text or
scalar.
}
\description{
Summaries are complicated objects because they reflect the Protobuf object
structure that are serialized in the tfevents records files. This function
allows one to easily query vaues from summaries and will dispatch to the
correct way to extract images, audio, text, etc from summary values.
}
\section{Methods (by class)}{
\itemize{
\item \code{value(tfevents_summary_values)}: Acess values from \code{summary_values}.

}}
\examples{
temp <- tempfile()
with_logdir(temp, {
  for(i in 1:5) {
    log_event(my_log = runif(1))
  }
})

# iterate over all events
summary <- collect_events(temp, n = 1, type = "summary")
value(summary$summary)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hparams.R
\name{hparams_hparam}
\alias{hparams_hparam}
\title{Defines a HParam}
\usage{
hparams_hparam(name, domain = NA, display_name = name, description = name)
}
\arguments{
\item{name}{Name of the hyperparameter.}

\item{domain}{A list of values that can be assumed by the hyperparameter.
It can be \code{character()}, \code{numeric()} or \code{logical()} vector. You can also
pass a named numeric vector with eg \code{c(min_value = 0, max_value = 10)} in
this case, any value in this range is accepted.}

\item{display_name}{Display name of the hparameter for the TensorBoard UI.
By default it's identical to the name.}

\item{description}{Parameter description. Shown in tooltips around the
TensorBoard UI.}
}
\value{
A \code{hparams_hparam} object.
}
\description{
Hparam object are used to describe names and domains of hyperparameters so
TensorBoard UI can show additional information about them.
}
\note{
A list of \code{hparam} values can be passed to \code{\link[=log_hparams_config]{log_hparams_config()}} so
you define the hyperparameters that are tracked by the experiment.
}
\examples{
hparams_hparam("optimizer", domain = c("adam", "sgd"))
hparams_hparam("num_units", domain = c(128, 512, 1024))
hparams_hparam("use_cnn", domain = c(TRUE, FALSE))
hparams_hparam("dropout", domain = c(min_value = 0, max_value = 0.5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/image.R
\name{summary_image}
\alias{summary_image}
\alias{summary_image.ggplot}
\alias{summary_image.array}
\alias{summary_image.blob}
\alias{summary_image.raw}
\title{Creates a image summary}
\usage{
summary_image(img, ..., metadata = NULL, tag = NA)

\method{summary_image}{ggplot}(img, ..., width = 480, height = 480, metadata = NULL, tag = NA)

\method{summary_image}{array}(img, ..., metadata = NULL, tag = NA)

\method{summary_image}{blob}(img, ..., width, height, colorspace, metadata = NULL, tag = NA)

\method{summary_image}{raw}(img, ..., width, height, colorspace, metadata = NULL, tag = NA)
}
\arguments{
\item{img}{An object that can be converted to an image.}

\item{...}{Currently unused.}

\item{metadata}{A \code{metadata} object, as created with \code{\link[=summary_metadata]{summary_metadata()}}. In
most cases you don't need to change the default.}

\item{tag}{A tag that within the TensorBoard UI. See \code{\link[=log_event]{log_event()}} for other
ways of specifying the tag attribute.}

\item{width}{Width of the image.}

\item{height}{Height of the image.}

\item{colorspace}{Valid colorspace values are
\code{1 - grayscale},
\code{2 - grayscale + alpha},
\code{3 - RGB},
\code{4 - RGBA},
\code{5 - DIGITAL_YUV},
\code{6 - BGRA}}
}
\value{
An image summary that can be logged with \code{\link[=log_event]{log_event()}}.
}
\description{
Creates a image summary
}
\section{Methods (by class)}{
\itemize{
\item \code{summary_image(ggplot)}: Cretes an image summary from a ggplot2 graph object.
The \code{...} will be forwarded to \code{\link[grDevices:png]{grDevices::png()}}.

\item \code{summary_image(array)}: Creates an image from an R array. The array should be
numeric, with values between 0 and 1. Dimensions should be \verb{(batch, height, width, channels)}.

\item \code{summary_image(blob)}: Creates an image from \code{\link[blob:blob]{blob::blob()}} vctr of PNG encoded images,
(eg using \code{\link[png:writePNG]{png::writePNG()}}). \code{width}, \code{height} and \code{colorspace} are recycled
thus they can be a single scalar or a vector the same size of the images blob.

\item \code{summary_image(raw)}: Creates an image from a png encoded image. Eg, created
with \code{\link[png:writePNG]{png::writePNG()}}. In this case you need to provide \code{width}, \code{height} and
\code{colorspace} arguments.

}}
\examples{
tmp <- tempfile()
with_logdir(tmp, {
  summary_image(array(runif(100), dim = c(1,10, 10, 1)))
})
}
\seealso{
Other summary: 
\code{\link{summary_audio}()},
\code{\link{summary_histogram}()},
\code{\link{summary_scalar}()},
\code{\link{summary_text}()}
}
\concept{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/log.R
\name{get_global_step}
\alias{get_global_step}
\alias{set_global_step}
\title{Global step counters}
\usage{
get_global_step(increment = TRUE)

set_global_step(step)
}
\arguments{
\item{increment}{Wether to increment the \code{step} when getting it.}

\item{step}{New value for \code{step}.}
}
\value{
The global step value for the default logdir, when \code{get_global_step},
otherwise returns \code{NULL} invisibly.
}
\description{
Global step counters
}
\details{
\code{tfevents} tracks and automatically increased the step counter whenever
\code{\link[=log_event]{log_event()}} is called. Note that, it maintains a separate step counter for
each root \code{logdir}, thus if you change the \code{logdir} using \code{\link[=set_default_logdir]{set_default_logdir()}}
or \code{\link[=with_logdir]{with_logdir()}}, a different step counter will be used.
}
\section{Functions}{
\itemize{
\item \code{set_global_step()}: Set the global step.

}}
\examples{
temp <- tempfile()
with_logdir(temp, {
  print(get_global_step())
  set_global_step(100)
  print(get_global_step())
})
print(get_global_step())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/log.R
\name{log_event}
\alias{log_event}
\title{Log event}
\usage{
log_event(..., step = get_global_step(increment = TRUE))
}
\arguments{
\item{...}{Named values that you want to log. They can be possibly nested,
in this case, the enclosing names are considered 'run' names by TensorBoard.}

\item{step}{The step associated the logs. If \code{NULL}, a managed step counter
will be used, and the global step is increased in every call to \code{\link[=log_event]{log_event()}}.}
}
\value{
Invisibly returns the logged data.
}
\description{
Log event
}
\note{
\code{\link[=log_event]{log_event()}} writes events to the default \code{logdir}. You can query the
default \code{logdir} with \code{\link[=get_default_logdir]{get_default_logdir()}} and modify it with
\code{\link[=set_default_logdir]{set_default_logdir()}}. You can also use the \code{\link[=with_logdir]{with_logdir()}} context switcher
to temporarily modify the logdir.
}
\examples{
temp <- tempfile()
with_logdir(temp, {
  log_event(
     train = list(loss = runif(1), acc = runif(1)),
     valid = list(loss = runif(1), acc = runif(1))
  )
})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hparams.R
\name{hparams_metric}
\alias{hparams_metric}
\title{Defines a Metric}
\usage{
hparams_metric(
  tag,
  group = NA,
  display_name = tag,
  description = tag,
  dataset_type = NA
)
}
\arguments{
\item{tag}{The tag name of the scalar summary that corresponds to this
metric.}

\item{group}{An optional string listing the subdirectory under the
session's log directory containing summaries for this metric.
For instance, if summaries for training runs are written to
events files in \code{ROOT_LOGDIR/SESSION_ID/train}, then \code{group}
should be \code{"train"}. Defaults to the empty string: i.e.,
summaries are expected to be written to the session logdir.}

\item{display_name}{An optional human-readable display name.}

\item{description}{An optional Markdown string with a human-readable
description of this metric, to appear in TensorBoard.}

\item{dataset_type}{dataset_type: Either \code{"training"} or \verb{"validation}, or
\code{NA}.}
}
\value{
A \code{hparams_metric} object.
}
\description{
Metric objects are passed to \code{\link[=log_hparams_config]{log_hparams_config()}} in order to define the
collection of scalars that will be displayed in the HParams tab in TensorBoard.
}
\examples{
hparams_metric("loss", group = "train")
hparams_metric("acc")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/log.R
\name{get_default_logdir}
\alias{get_default_logdir}
\alias{set_default_logdir}
\alias{with_logdir}
\alias{local_logdir}
\title{Query and modify the logdir}
\usage{
get_default_logdir()

set_default_logdir(logdir = "logs")

with_logdir(logdir, code)

local_logdir(logdir, .env = parent.frame())
}
\arguments{
\item{logdir}{The \code{logdir} that you want to set as default.}

\item{code}{Expressions that will be evaluated in a context with the \code{new}
\code{logdir} as the default \code{logdir}.}

\item{.env}{Environment that controls scope of changes. For expert use only.}
}
\value{
The \code{logdir} for \code{get_default_logdir()} otherwise invisibly returns
\code{NULL}
}
\description{
\code{\link[=log_event]{log_event()}} has a notion of default logdir, so you don't need to specify it
at every call. These functions allow you to query and the current logdir.
}
\section{Functions}{
\itemize{
\item \code{set_default_logdir()}: Modifies the default \code{logdir}.

\item \code{with_logdir()}: Temporarily modify the default \code{logdir}.

\item \code{local_logdir()}: Temporarily modify thedefault \code{logdir}.

}}
\examples{
temp <- tempfile()
get_default_logdir()
with_logdir(temp, {
 print(get_default_logdir())
})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audio.R
\name{summary_audio}
\alias{summary_audio}
\alias{summary_audio.array}
\alias{summary_audio.raw}
\alias{summary_audio.blob}
\title{Summary audio}
\usage{
summary_audio(audio, ..., metadata = NULL, tag = NA)

\method{summary_audio}{array}(audio, ..., sample_rate = 44100, metadata = NULL, tag = NA)

\method{summary_audio}{raw}(audio, ..., metadata = NULL, tag = NA)

\method{summary_audio}{blob}(audio, ..., metadata = NULL, tag = NA)
}
\arguments{
\item{audio}{Object that will be written as an audio event in the tfevents record.}

\item{...}{Currently unused.}

\item{metadata}{A \code{metadata} object, as created with \code{\link[=summary_metadata]{summary_metadata()}}. In
most cases you don't need to change the default.}

\item{tag}{A tag that within the TensorBoard UI. See \code{\link[=log_event]{log_event()}} for other
ways of specifying the tag attribute.}

\item{sample_rate}{The sample rate in Hz associated to the audio values.}
}
\value{
An audio summary that can be logged with \code{\link[=log_event]{log_event()}}.
}
\description{
Audio summaries can be played withing the TensorBoard UI.
}
\section{Methods (by class)}{
\itemize{
\item \code{summary_audio(array)}: Creates a summary from a 3D array with dimensions
\verb{(batch_size, n_samples, n_channels)}. Values must be in the range \verb{[-1, 1]}.

\item \code{summary_audio(raw)}: Creates an audio summary from a raw vector containing
a WAV encoded audio file.

\item \code{summary_audio(blob)}: Creates an audio summary from a blob (ie list of raw vectors)
containing WAV encoded audio files.

}}
\examples{
\dontshow{if (rlang::is_installed("wav")) withAutoprint(\{ # examplesIf}
tmp <- tempfile()
with_logdir(tmp, {
  summary_audio(array(runif(100), dim = c(1,100, 1)))
})
\dontshow{\}) # examplesIf}
}
\seealso{
Other summary: 
\code{\link{summary_histogram}()},
\code{\link{summary_image}()},
\code{\link{summary_scalar}()},
\code{\link{summary_text}()}
}
\concept{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events.R
\name{as_event}
\alias{as_event}
\title{Coerce an object to a event.}
\usage{
as_event(x, step, wall_time, ...)
}
\arguments{
\item{x}{Object that will be coerced to an event.}

\item{step}{The step that will be used when the event is logged. This is used
by TensorBoard when showing data.}

\item{wall_time}{The all time the event will appended to the event. This field
is used by TensorBoard when displaying information based on actual time.}

\item{...}{currently unused.}
}
\value{
A event vctr with class <tfevents_event>.
}
\description{
Coerce an object to a event.
}
\section{Extending \code{as_event}}{


\code{as_event} is an S3 generic and you can implement method for your own class.
We don't export the \code{event} constructor though, so you should implement it
in terms of other \code{as_event} methods.
}

\examples{
as_event(list(hello = 1), step = 1, wall_time = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.R
\name{collect_events}
\alias{collect_events}
\alias{events_logdir}
\title{Collect data from tfevents records}
\usage{
collect_events(
  logdir = get_default_logdir(),
  n = NULL,
  type = c("any", "summary", "scalar")
)

events_logdir(logdir = get_default_logdir())
}
\arguments{
\item{logdir}{The log directory that you want to query events from. Either a
file path or a connection created with \code{\link[=events_logdir]{events_logdir()}}.}

\item{n}{The maximum number of events to read from the connection. If \code{NULL}
then all events are read, the default is \code{NULL}.}

\item{type}{The kind of events that are to be read. By default all events are
read. If a different type is specified, then the result can include other
columns as well as more lines.}
}
\value{
A \code{tibble} with the collected events.
}
\description{
Collects all events of a kind in a single data.frame ready for analysis.
}
\section{Functions}{
\itemize{
\item \code{events_logdir()}: Creates a connection to a logdir that can be reused
to read further events later.

}}
\examples{
temp <- tempfile()
with_logdir(temp, {
  for(i in 1:5) {
    log_event(my_log = runif(1))
  }
})
# collect all events in files, including file description events
collect_events(temp)
# collect summaries in the logdir
collect_events(temp, type = "summary")
# collect only scalar events
collect_events(temp, type = "scalar")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hparams.R
\name{log_hparams_config}
\alias{log_hparams_config}
\alias{summary_hparams_config}
\title{Logs hyperparameters configuration}
\usage{
log_hparams_config(hparams, metrics, time_created_secs = get_wall_time())

summary_hparams_config(hparams, metrics, time_created_secs = get_wall_time())
}
\arguments{
\item{hparams}{A list of \code{hparams} objects as created by \code{\link[=hparams_hparam]{hparams_hparam()}}.}

\item{metrics}{A list of \code{metrics} objects as created by \code{\link[=hparams_metric]{hparams_metric()}}.
These metrics will be tracked by TensorBoard UI when displaying the
hyperparameter tuning table.}

\item{time_created_secs}{The time the experiment is created in seconds
since the UNIX epoch.}
}
\value{
Invisibly returns the HParam conffuration data as a \code{summary} object.
}
\description{
Logs the hyperaparameter configuration for a HyperParameter tuning experiment.
It allows you to define the domain for each hyperparameters and what are the
metrics that should be shown in the TensorBoard UI, along with configuring
their display name and descriptions.
}
\section{Functions}{
\itemize{
\item \code{summary_hparams_config()}: For advanced users only. Creates a hyperaparameter
configuration summary that can be logged with \code{\link[=log_event]{log_event()}}.

}}
\section{Recommendations}{


When loging hyperparameter tuning experiments, the log directory organization
is:

\if{html}{\out{<div class="sourceCode">}}\preformatted{- root:
 - log_hparams_config(...)
 - run1:
   - log_hparams(...)
   - log_event(...)
 - run2:
   - log_hparams(...)
   - log_event(...)
}\if{html}{\out{</div>}}

Ie you should have a root logdir that will only contain the hyperaparameter
config log, as created with \code{\link[=log_hparams_config]{log_hparams_config()}}. Then each run in the
experiment will have it's own logdir as a child directory of the root logdir.
}

\seealso{
\code{\link[=log_hparams]{log_hparams()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events.R
\name{event}
\alias{event}
\title{Creates events}
\usage{
event(run, wall_time, step, ..., summary = NA, file_version = NA)
}
\description{
We try to match events as closely as possible to the protobuf messages.
The hierarchy looks like:

\if{html}{\out{<div class="sourceCode">}}\preformatted{event (<event>):
 - run (<character>)
 - wall_time (<integer>)
 - step (<integer>)
 - summary (<summary> aka list_of<summary_values>):
    - values (list):
      - <summary_value>:
        - metadata (<summary_metadata>)
        - tag (<character>)
        - value (<numeric>)
        - image (<summary_summary_image>)
          - buffer (<blob>)
          - width (<integer>)
          - height (<integer>)
          - colorspace (<integer>)
}\if{html}{\out{</div>}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/text.R
\name{summary_text}
\alias{summary_text}
\alias{summary_text.character}
\title{Creates a text summary}
\usage{
summary_text(txt, ..., metadata = NULL, tag = NA)

\method{summary_text}{character}(txt, ..., metadata = NULL, tag = NA)
}
\arguments{
\item{txt}{An object that can be converted to a text.}

\item{...}{Currently unused.}

\item{metadata}{A \code{metadata} object, as created with \code{\link[=summary_metadata]{summary_metadata()}}. In
most cases you don't need to change the default.}

\item{tag}{A tag that within the TensorBoard UI. See \code{\link[=log_event]{log_event()}} for other
ways of specifying the tag attribute.}
}
\value{
A summary that can be logged with \code{\link[=log_event]{log_event()}}.
}
\description{
Creates a text summary
}
\section{Methods (by class)}{
\itemize{
\item \code{summary_text(character)}: Creates a summary from a scalar character vector.

}}
\examples{
temp <- tempfile()
with_logdir(temp, {
  log_event(
    x = "hello world",
    y = summary_text("hello world")
  )
})
}
\seealso{
Other summary: 
\code{\link{summary_audio}()},
\code{\link{summary_histogram}()},
\code{\link{summary_image}()},
\code{\link{summary_scalar}()}
}
\concept{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scalar.R
\name{summary_scalar}
\alias{summary_scalar}
\title{Scalar event}
\usage{
summary_scalar(value, ..., metadata = NULL, tag = NA)
}
\arguments{
\item{value}{A numeric scalar value to be logged.}

\item{...}{Currently unused. To allow future expansion.}

\item{metadata}{A \code{metadata} object, as created with \code{\link[=summary_metadata]{summary_metadata()}}. In
most cases you don't need to change the default.}

\item{tag}{A tag that within the TensorBoard UI. See \code{\link[=log_event]{log_event()}} for other
ways of specifying the tag attribute.}
}
\value{
A \verb{<scalar_event>} object.
}
\description{
Scalar event
}
\examples{
temp <- tempfile()
with_logdir(temp, {
  log_event(loss = summary_scalar(1))
})
}
\seealso{
Other summary: 
\code{\link{summary_audio}()},
\code{\link{summary_histogram}()},
\code{\link{summary_image}()},
\code{\link{summary_text}()}
}
\concept{summary}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hparams.R
\name{log_hparams}
\alias{log_hparams}
\alias{summary_hparams}
\title{Log hyperaparameters}
\usage{
log_hparams(..., trial_id = NA, time_created_secs = get_wall_time())

summary_hparams(..., trial_id = NA, time_created_secs = get_wall_time())
}
\arguments{
\item{...}{Named values of hyperparameters.}

\item{trial_id}{A name for the current trail. by default it's the hash of the
hparams names and values.}

\item{time_created_secs}{The time the experiment is created in seconds
since the UNIX epoch.}
}
\value{
A hyperparameter summary. USed for the side effects of logging the
hyperparameter to the logdir.
}
\description{
Log hyperaparameters
}
\details{
This function should only be called once in a logdir and it will
record the set of hyperparameters used in that run. Undefined behavior can
happen if it's called more than once in a logdir - specially how TensorBoard
behaves during visualization.
}
\section{Functions}{
\itemize{
\item \code{summary_hparams()}: For advanced users only. It's recommended to use the \code{log_hparams()}
function instead. Creates a hyperparameter summary that can be written with \code{log_event()}.

}}
\examples{
temp <- tempfile()
with_logdir(temp, {
  log_hparams(optimizer = "adam", num_units = 16)
})
}
\seealso{
\code{\link[=log_hparams_config]{log_hparams_config()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/events.R
\name{summary_metadata}
\alias{summary_metadata}
\title{Summary metadata}
\usage{
summary_metadata(
  plugin_name,
  display_name = NA_character_,
  description = NA_character_,
  ...,
  plugin_content = NA
)
}
\arguments{
\item{plugin_name}{The name of the TensorBoard plugin that might use the summary.}

\item{display_name}{Display name for the summary.}

\item{description}{A description of the summary.}

\item{...}{Currently unused. For future expansion.}

\item{plugin_content}{An optional plugin content. Note that it will only be
used if the C++ function \code{make_plugin_data} is aware of \code{plugin_content}
for the specified plugin name. For advanced use only.}
}
\value{
A \code{summary_metadata} object.
}
\description{
Creates a summary metadata that can be passed to multiple \code{summary_} functions.
}
\examples{
summary <- summary_scalar(1, metadata = summary_metadata("scalars"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histogram.R
\name{summary_histogram}
\alias{summary_histogram}
\alias{summary_histogram.numeric}
\alias{summary_histogram.array}
\title{Creates an histogram summary}
\usage{
summary_histogram(data, ..., metadata = NULL, tag = NA)

\method{summary_histogram}{numeric}(data, ..., metadata = NULL, tag = NA, buckets = 30)

\method{summary_histogram}{array}(data, ..., metadata = NULL, tag = NA, buckets = 30)
}
\arguments{
\item{data}{A Tensor of any shape. The histogram is computed over its elements,
which must be castable to float64.}

\item{...}{Currently unused. To allow future expansion.}

\item{metadata}{A \code{metadata} object, as created with \code{\link[=summary_metadata]{summary_metadata()}}. In
most cases you don't need to change the default.}

\item{tag}{A tag that within the TensorBoard UI. See \code{\link[=log_event]{log_event()}} for other
ways of specifying the tag attribute.}

\item{buckets}{Optional positive int. The output will have this many buckets,
except in two edge cases. If there is no data, then there are no buckets.
If there is data but all points have the same value, then all buckets' left
and right endpoints are the same and only the last bucket has nonzero count.
Defaults to 30 if not specified.}
}
\value{
An histogram summary that can be logged with \code{\link[=log_event]{log_event()}}.
}
\description{
Writes an histogram for later analysis in TensorBoard's Histograms and
Distributions tab.
}
\section{Methods (by class)}{
\itemize{
\item \code{summary_histogram(numeric)}: Creates an histogram summary for a numeric vector.

\item \code{summary_histogram(array)}: Creates an histogram for array data.

}}
\examples{
temp <- tempfile()
with_logdir(temp, {
  for(i in 1:10) {
    log_event(x = summary_histogram(rnorm(10000)))
  }
})

}
\seealso{
Other summary: 
\code{\link{summary_audio}()},
\code{\link{summary_image}()},
\code{\link{summary_scalar}()},
\code{\link{summary_text}()}
}
\concept{summary}
